#ifndef __PDE_BAS_H__
#define __PDE_BAS_H__

#ifndef __PDE_UTL_H__
#	include "pde_utl.h"
#endif

class AFX_EXT_CLASS CMSIndepVar;

class AFX_EXT_CLASS CPDEBase : public CPElem {
	DECLARE_ELEM(CPDEBase)
public:
	CPDEBase();
	CPDEBase *GetBaseObj();
	virtual int CalcExtVert(dirMode dir=dirUp, int mg=0);
	virtual int CalcExtMidVert(dirMode dir=dirUp, int mg=0);
	virtual int CalcExtHTab(dirMode dir=dirUp, int tabWd=50);
	virtual int CalcExtLines(dirMode dir=dirUp, int wd=10, int hg=10);
	virtual int CalcExtHMerg(dirMode dir, const char *nm, int mg);
	virtual void OnDrawName(CDC *, const CPoint&);
	virtual void OnDrawTest(CDC *pdc, const CPoint& p);
	virtual void OnDrawRel(CDC *pdc, const CPoint& p);
	virtual BOOL SendMsg(int msg, CObject* el);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
	CIElem * GetNewInst();
	void OnDrawClose(CDC *, const CPoint&);
};

class AFX_EXT_CLASS CPDEIElem : public CIElem {
	DECLARE_ELEM(CPDEIElem)
public:
	virtual BOOL PackInst();
};


class AFX_EXT_CLASS CPDEIndexFold : public CPDEBase {
	DECLARE_ELEM(CPDEIndexFold)
public:
	CPDEIndexFold();

	virtual BOOL SendMsg(int msg, CObject* el);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDESet : public CPDEBase {
	DECLARE_ELEM(CPDESet)
public:
	virtual BOOL SendMsg(int msg, CObject* el);
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDESpaceVar : public CPDEBase {
	DECLARE_ELEM(CPDESpaceVar)
public:
	CMSIndepVar *m_pIndepVar;
	CMSIndep *m_pIndep;
	CMSIndex *m_pIndex;
	CMSCInteger *m_division;

	CPDESpaceVar();
	BOOL CreateSpace(const char* ind, const char* dx, const char *I, const char *IMAX, int no);
	BOOL CreateSpaceConst(const char* ind, const char* dx, const char *I, const char *IMAX, int no);
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual int OnCommand(UINT);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *, const CPoint&);
	virtual BOOL SendMsg(int, CObject*);
};

class AFX_EXT_CLASS CPDETimeVar : public CPDESpaceVar {
	DECLARE_ELEM(CPDETimeVar)
public:
	CPDEIndexFold *m_indexFold;

	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *, const CPoint&);
	BOOL CreateTimeVar(const char* ind, const char* dx, const char *I, const char *IMAX, int no);
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
};

class AFX_EXT_CLASS CPDESpaceVars : public CPDEBase {
	DECLARE_ELEM(CPDESpaceVars)
public:
	CPDESpaceVars();
	int m_spaceNo;
	CPDESpaceVar *m_spaceVar[3];

	virtual BOOL SendMsg(int, CObject*);
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
};

class AFX_EXT_CLASS CPDEProcess : public CPDEBase {
	DECLARE_ELEM(CPDEProcess)
public:
	CPDEProcess();
	CViecFC *MakeCopy(CViecFC *pelm);
	BOOL GetInputOutput(CViecFC*& pin, CViecFC*& pout);
	CViecFC *CopyProc();

	virtual BOOL SendMsg(int, CObject*);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int OnAccess(CPoint *pp);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEPrcManip : public CPDEProcess {
	DECLARE_ELEM(CPDEPrcManip)
public:
	virtual int OnAccess(CPoint *pp);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEPrcDisc : public CPDEProcess {
	DECLARE_ELEM(CPDEPrcDisc)
public:
	virtual int OnAccess(CPoint *pp);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEObject : public CPDEBase {
	DECLARE_ELEM(CPDEObject)
public:
	CPDEObject();
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int OnAccess(CPoint *pp);
	virtual int OnCommand(UINT);
	virtual BOOL SendMsg(int, CObject*);
};

class AFX_EXT_CLASS CPDEPosDef : public CPDEBase {
protected:
	DECLARE_ELEM(CPDEPosDef)
public:
	BYTE m_pos[3];
	BYTE m_velocity;

	CPDEPosDef();
	virtual int OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	virtual void CopyBody(CViecFC *);
	virtual void SerializeBody(CArchive& ar);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode d=dirUp);
};

class AFX_EXT_CLASS CMSDepVar : public CMSDep {
	DECLARE_ELEM(CMSDepVar)
public:
	virtual BOOL SendMsg(int msg, CObject* el);
};

class AFX_EXT_CLASS CPDEDepInfo : public CPDEBase {
	DECLARE_ELEM(CPDEDepInfo)
public:
	CPDEDepInfo();
	CViecFC *m_descript;
	CMSDepVar *m_var;
	CPDEPosDef *m_pos;

	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual BOOL SendMsg(int msg, CObject* el);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);

	BOOL CreateDep1D(const char *nm, const char *var, int s1, int s2);
	BOOL CreateDep2D(const char *nm, const char *var, int s1, int s2);
	BOOL CreateDep3D(const char *nm, const char *var, int s1, int s2);
};

class AFX_EXT_CLASS CPDEParaInfo : public CPDEBase {/*CPDEParaInfo mean Parameter Information.*/
	DECLARE_ELEM(CPDEParaInfo)
public:
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int OnAccess(CPoint *p = NULL);
	virtual BOOL SendMsg(int, CObject*);
};

class AFX_EXT_CLASS CPDEFileFold : public CPDEParaInfo {
	DECLARE_ELEM(CPDEFileFold)
public:
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT id);
};

class AFX_EXT_CLASS CPDEDepPDE : public CPDEBase {
	DECLARE_ELEM(CPDEDepPDE)
public:
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	BOOL CreateDepPDE();
	virtual int OnAccess(CPoint *p = NULL);
};

class AFX_EXT_CLASS CPDEBoundCond : public CSymbInst {
	DECLARE_ELEM(CPDEBoundCond)
public:
	CPDEBoundCond() : CSymbInst() {};
	CPDEBoundCond(const char* nm) : CSymbInst(nm) {};
public:
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEBoundConds : public CPDEBase {
	DECLARE_ELEM(CPDEBoundConds)
public:
public:
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode dir=dirUp);
};

class AFX_EXT_CLASS CPDETarget : public CSymbInst {
protected:
	DECLARE_ELEM(CPDETarget)
public:
	CPDETarget() : CSymbInst() {};
	CPDETarget(const char *pi) : CSymbInst(pi) {};
};

class AFX_EXT_CLASS CPDESolvSet : public CPDESet {
protected:
	DECLARE_ELEM(CPDESolvSet)
public:
	virtual BOOL OnCreated(EnumOnCreated = SKIP_IF_CREATED);
	virtual BOOL SendMsg(int msg, CObject* el);
};

class AFX_EXT_CLASS CPDERunSpread : public CPDEBase {	 // used during program designing
	DECLARE_ELEM(CPDERunSpread)
public:
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
	BOOL CreateNewTemplate(CViecFC *pelm);
};

class AFX_EXT_CLASS CPDEAbstProcVis : public CPDEBase {
	DECLARE_ELEM(CPDEAbstProcVis)
public:
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEObtainedVars : public CPDEBase {
	DECLARE_ELEM(CPDEObtainedVars)
public:
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEInvokedProcs : public CPDEBase {
	DECLARE_ELEM(CPDEInvokedProcs)
public:
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDELineTerm: public CPDEBase {
	DECLARE_ELEM(CPDELineTerm)
public:
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
};

class AFX_EXT_CLASS CPDEMtxDefs : public CPDEBase {
	DECLARE_ELEM(CPDEMtxDefs)
public:
	CPDEMtxDefs();
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
};

class AFX_EXT_CLASS CPDEBrkPntDefs : public CPDEMtxDefs {
	DECLARE_ELEM(CPDEBrkPntDefs)
public:
	virtual int OnAccess(CPoint *pp);
	virtual int OnCommand(UINT);
};

class AFX_EXT_CLASS CPDEConstDef : public CMSIndex {
	DECLARE_ELEM(CPDEConstDef)
public:
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
};

class AFX_EXT_CLASS CMSIndepVar : public CMSIndep {
	DECLARE_ELEM(CMSIndepVar)
public:
	virtual BOOL SendMsg(int msg, CObject* el);
};

class AFX_EXT_CLASS CMSIndepConst : public CMSIndep {
	DECLARE_ELEM(CMSIndepConst)
protected:
	double m_value;
public:
	CMSIndepConst(double val = 1.0);
	double GetValue() { return m_value; };
	virtual BOOL SendMsg(int msg, CObject* el);
	virtual int OnAccess(CPoint *p = NULL);
	virtual int OnCommand(UINT);
	virtual int CalcExt(dirMode dir=dirUp);
	virtual void OnDraw(CDC *pdc, const CPoint& p);
	virtual void SerializeBody(CArchive& ar);
};

#endif __PDE_BAS_H__
