#include "stdafx.h"
#include <math.h>

#include "pde_utl.h"
#include "pde_msg.h"

#include "realdlg.h"

IMPLEMENT_ELEM(CPDEBttn, CViecFC, 265)
IMPLEMENT_ELEM(CMSCInteger, CMSConst, 212)
IMPLEMENT_ELEM(CMSCReal, CMSConst, 213)
IMPLEMENT_ELEM(CMSCRealSub, CMSCReal, 213)
IMPLEMENT_ELEM(CMSFract, CMSNumber, 261)
IMPLEMENT_ELEM(CMSRealEx, CMSNumber, 273)
IMPLEMENT_ELEM(CMSPDEMtx, CMSMatrix, 238)
IMPLEMENT_ELEM(CMSPDEVct, CMSPDEMtx, 239)
IMPLEMENT_ELEM(CEFile, CViecFC, 275)
IMPLEMENT_ELEM(CEFileFrom, CViecFC, 276)

//////////////// CMSCInteger ////////////////////
//////////////// CMSCInteger ////////////////////

void CMSCInteger::OnDraw(CDC *pdc, const CPoint& p)
{
	TCHAR buf[40];
	sprintf(buf, "%s = %d", (const char*) GetName(), m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(buf, lstrlen(buf));
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);
	CPoint p0 = p;
	p0.Offset(GetOrg());
	TextOut(pdc, p0.x, p0.y, buf, lstrlen(buf));
}

int CMSCInteger::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	CDC *pdc = GetDC();
	if(pdc==NULL) return FALSE;
	CString str;
	str.Format("%s = %d", (const char*) GetName(), m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	m_extent = GetTextExtent(pdc, str, lstrlen(str));
	ReleaseDC(pdc);
	CalcExtPost(dir, ext);
	return TRUE;
}


int CMSCInteger::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch(nChar) {
	case VK_UP:
		m_value += 1;
		GetParent()->SendMsg(MSG_CONST_CHANGED, this);
//		GetAncestor()->SendMsg(MSG_PDE_MESH_CHANGED, NULL);
		m_extent.right++;
		CalcExt();
//		InvalidateRect(CPoint(0,0));
		return TRUE;
	case VK_DOWN:
		m_value -= 1;
		GetParent()->SendMsg(MSG_CONST_CHANGED, this);
//		GetAncestor()->SendMsg(MSG_PDE_MESH_CHANGED, NULL);
		m_extent.right++;
		CalcExt();
		return TRUE;
	}
	return CMSConst::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMSCInteger::CopyBody(CViecFC *pelm)
{
	CMSCInteger *pCp = (CMSCInteger *) pelm;
	CMSConst::CopyBody(pelm);
	pCp->m_value = m_value;
}

void CMSCInteger::SerializeBody(CArchive& ar)
{
	CMSConst::SerializeBody(ar);
	if(ar.IsStoring()) {
		ar << (DWORD) m_value;
	} else {
		DWORD u;
		ar >> u;
		m_value = u;
	}
}

///////////////////   CMSCReal   /////////////////////
///////////////////   CMSCReal   /////////////////////

int CMSCReal::CalcExt(dirMode dir)
{
	TCHAR buf[40];
	CRect ext = m_extent;
	CDC *pdc = GetDC();
	if(pdc==NULL) return FALSE;
	sprintf(buf, "%s = %10.4e", (const char*) GetName(), m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	m_extent = GetTextExtent(pdc, buf, lstrlen(buf));
	ReleaseDC(pdc);
	CalcExtPost(dir, ext);
	return TRUE;
}

void CMSCReal::OnDraw(CDC *pdc, const CPoint& p)
{
	TCHAR buf[40];
	sprintf(buf, "%s = %10.4e", (const char*) GetName(), m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);
	CPoint p0 = p;
	p0.Offset(GetOrg());
	TextOut(pdc, p0.x, p0.y, buf, lstrlen(buf));
}

int CMSCReal::OnAccess(CPoint *pp)
{
	CMenu *pMenu = GetPopupMenu(TRUE);
	CString str = GetRuntimeClass()->m_lpszClassName;
	pMenu->AppendMenu(MF_STRING, 0, str+": "+GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	CMenu *psub = GetPopupMenu(FALSE);

	pMenu->AppendMenu(MF_STRING, CMD_CHANG_NAME, "Change &Name");
	pMenu->AppendMenu(MF_STRING, CMD_RECAL_SIZE, "&Recalc Size");
	pMenu->AppendMenu(MF_STRING, CMD_INPUT_VALUE, "&input value");
	if(GetParent()->SendMsg(MSG_CAN_THIS_DISAPPEAR, this))
		pMenu->AppendMenu(MF_STRING, CMD_ADD_DELETE, "&Delete");
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

int CMSCReal::OnCommand(UINT id)
{
	switch(id) {
	case CMD_INPUT_VALUE:
		{
			CRealValueDlg dlg;
			dlg.m_value = m_value;
			dlg.m_label = GetName();
			if(dlg.DoModal()==IDOK) {
				m_value = dlg.m_value;
				m_extent.right++;
				GetParent()->SendMsg(MSG_CONST_CHANGED, this);
				CalcExt();
			}
		}
		return TRUE;
	}
	return CMSConst::OnCommand(id);
}

void CMSCReal::CopyBody(CViecFC *pelm)
{
	CMSCReal *pCp = (CMSCReal *) pelm;
	CMSConst::CopyBody(pelm);
	pCp->m_value = m_value;
}

void CMSCReal::SerializeBody(CArchive& ar)
{
	CMSConst::SerializeBody(ar);
	if(ar.IsStoring()) {
		ar << m_value;
	} else {
		ar >> m_value;
	}
}

///////////////////   CMSCRealSub   /////////////////////
///////////////////   CMSCRealSub   /////////////////////

int CMSCRealSub::CalcExt(dirMode dir)
{
	TCHAR buf[40];
	CRect ext = m_extent;
	CDC *pdc = GetDC();
	if(pdc==NULL) return FALSE;
	sprintf(buf, "%6.3f", m_value);
	// sprintf(buf, "%6.3e", m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	m_extent = GetTextExtent(pdc, buf, lstrlen(buf));
	ReleaseDC(pdc);
	CalcExtPost(dir, ext);
	return TRUE;
}

void CMSCRealSub::OnDraw(CDC *pdc, const CPoint& p)
{
	TCHAR buf[40];
	sprintf(buf, "%6.3f", m_value);
	// sprintf(buf, "%6.3e", m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);
	CPoint p0 = p;
	p0.Offset(GetOrg());
	TextOut(pdc, p0.x, p0.y, buf, lstrlen(buf));
}

/////////////////////////   CMSFract  /////////////////////////
/////////////////////////   CMSFract  /////////////////////////

int CMSFract::CalcExt(dirMode dir)
{
	CRect ext = m_extent;

	CSymbInst *inst = (CSymbInst *) GetParent();
	if(inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
		CViecFC *pSym = inst->GetSymbBody();
		if(pSym->IsKindOf(RUNTIME_CLASS(CMSONeg))) {
			m_value = - m_value;
			CViecFC *pelm = inst->GetParent();
			pelm->SubstChild(inst, this, dirDown);
			GetView()->CalcExt(dirDown);
			inst->m_child.RemoveTail();
			delete inst;
			inst = (CSymbInst *) GetParent();
		}
	}

	int lv = 0;
	if(inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
		lv = inst->GetLevel();
	}

	CString str1, str2;
	CDC *pdc = GetDC();
	if(pdc==NULL) return FALSE;
//	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	SetFont(pdc, GetnDot(lv, m_nDot), GetFont(), m_nCharSet, m_nItalic);

	if(m_deno>1 && m_value!=0) {
		str1.Format("%d", m_value);
		str2.Format("%d", m_deno);
		// str1.Format("%6.3f", m_value);
		// str2.Format("%6.3f", m_deno);
		if(m_value<0 && inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
			CViecFC *psym = inst->GetSymbBody();
			if(psym->IsKindOf(RUNTIME_CLASS(CMSOPlus)))
				str1.Format("%d", -m_value);
		}
		CSize sz1 = pdc->GetTextExtent(str1, lstrlen(str1));
		CSize sz2 = pdc->GetTextExtent(str2, lstrlen(str2));
		CRect ext = m_extent;
		m_extent.left = 0;
		m_extent.right = sz1.cx>sz2.cx? sz1.cx + 4: sz2.cx + 4;
		m_extent.top = - sz1.cy - 2;
		m_extent.bottom = sz2.cy + 2;
	} else {
		CString str;
		if(m_value<0)	// str.Format("(%6.3f)", m_value);
						str.Format("(%d)", m_value);
		else			// str.Format("%6.3f", m_value);
						str.Format("%d", m_value);
		if(m_value<0 && inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
			CViecFC *psym = inst->GetSymbBody();
			if(psym->IsKindOf(RUNTIME_CLASS(CMSOPlus)))
				// str.Format("%6.3f", -m_value);
				str.Format("%d", -m_value);
		}
		CSize sz = pdc->GetTextExtent(str, lstrlen(str));
		m_extent.left = 0;
		m_extent.right = sz.cx;
		m_extent.top = -sz.cy/2;
		m_extent.bottom = sz.cy + m_extent.top;
	}
	ReleaseDC(pdc);
	if(dir==dirUp && m_parent)
		m_parent->CalcExt(dir);
	if(m_extent!=ext) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
		return 1;
	}
	return 0;
}

void CMSFract::OnDraw(CDC *pdc, const CPoint& p)
{
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	CPoint p0=p;
	p0.Offset(m_org);
	int x1, x2, y1, y2;
	CSymbInst *inst = (CSymbInst *) GetParent();

	int lv = 0;
	if(inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
		lv = inst->GetLevel();
	}

	SetFont(pdc, GetnDot(lv, m_nDot), GetFont(), m_nCharSet, m_nItalic);
	if(m_deno>1 && m_value!=0) {
		CString str1, str2;
		str1.Format("%d", m_value);
		str2.Format("%d", m_deno);
		// str1.Format("%6.3f", m_value);
		// str2.Format("%6.3f", m_deno);
		if(m_value<0 && inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
			CViecFC *psym = inst->GetSymbBody();
			if(psym->IsKindOf(RUNTIME_CLASS(CMSOPlus)))
				str1.Format("%d", -m_value);
				// str1.Format("%6.3f", -m_value);
		}
		CSize sz1 = pdc->GetTextExtent(str1, lstrlen(str1));
		CSize sz2 = pdc->GetTextExtent(str2, lstrlen(str2));
		x1 = m_extent.right/2 - (sz1.cx+1)/2;
		x2 = m_extent.right/2 - (sz2.cx+1)/2;
		y1 = - sz1.cy;
		y2 = 0;
		pdc->SetTextColor(col);
		pdc->TextOut(p0.x+x1, p0.y+y1, str1);
		pdc->TextOut(p0.x+x2, p0.y+y2, str2);
		SetPen(pdc, PS_SOLID, 1, col);
		pdc->MoveTo(p0.x, p0.y);
		pdc->LineTo(p0.x+m_extent.right, p0.y);
	} else {
		CString str;
		if(m_value<0)	str.Format("(%d)", m_value);
		else			str.Format("%d", m_value);
						//str.Format("%6.3f", m_value);

		if(m_value<0 && inst->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
			CViecFC *psym = inst->GetSymbBody();
			if(psym->IsKindOf(RUNTIME_CLASS(CMSOPlus)))
				str.Format("%d", -m_value);
				// str.Format("%6.3f", -m_value);
		}
		pdc->SetTextColor(col);
		CSize sz = pdc->GetTextExtent(str, lstrlen(str));
		pdc->TextOut(p0.x, p0.y-sz.cy/2, str);
	}
}

void CMSFract::gdc()
{
	unsigned int a,b,c;
	if(m_value<0) a = - m_value;
	else a = m_value;
	b = m_deno;
	if(a<b) { c = a; a = b; b = c; }
	while(b>1) {
		c = a % b;
		if(c==0) break;
		a = b;
		b = c;
	}
	if(b<=1) return;
	m_value /= (int) b;
	m_deno /= b;
}

int CMSFract::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch(nChar) {
	case VK_UP:
		switch(MyGetKeyState()) {
		case SS_NOTHING:
			m_value += 1;
			gdc();
			CalcExt();
			InvalidateRect(CPoint(0,0));
			return TRUE;
		case SS_SHIFT:
			m_deno += 1;
			gdc();
			CalcExt();
			InvalidateRect(CPoint(0,0));
			return TRUE;
		}
	case VK_DOWN:
		switch(MyGetKeyState()) {
		case SS_NOTHING:
			m_value -= 1;
			gdc();
			CalcExt();
			InvalidateRect(CPoint(0,0));
			return TRUE;
		case SS_SHIFT:
			if(m_deno<=1) return TRUE;
			m_deno -= 1;
			gdc();
			CalcExt();
			InvalidateRect(CPoint(0,0));
			return TRUE;
		}
	}
	return CMSNumber::OnKeyDown(nChar, nRepCnt, nFlags);
}

BOOL CMSFract::IsEqual(int nume, unsigned int deno)
{
	if(m_value==nume && deno==m_deno)
		return TRUE;
	return FALSE;
}

BOOL CMSFract::Mult(int nume, unsigned int deno)
{
	if(deno<1) return FALSE;
	m_value *= nume;
	m_deno *= deno;
	gdc();
	return TRUE;
}

BOOL CMSFract::Plus(int nume, unsigned int deno)
{
	if(m_deno==deno) {
		m_value += nume;
	} else {
		m_value = m_value * deno + nume * m_deno;
		m_deno = m_deno * deno;
	}
	gdc();
	return TRUE;
}

BOOL CMSFract::Plus(int nume)
{
	if(m_deno==1) {
		m_value += nume;
	} else {
		m_value = m_value + nume * m_deno;
	}
	gdc();
	return TRUE;
}

void CMSFract::CopyBody(CViecFC *pelm)
{
	CMSFract *pCp = (CMSFract *) pelm;
	CMSNumber::CopyBody(pelm);
	pCp->m_value = m_value;
	pCp->m_deno = m_deno;
}

void CMSFract::SerializeBody(CArchive& ar)
{
	CMSNumber::SerializeBody(ar);
	if(ar.IsStoring()) {
		ar << (DWORD) m_value;
		ar << (DWORD) m_deno;
	} else {
		DWORD u;
		ar >> u;
		m_value = u;
		ar >> u;
		m_deno = u;
	}
}

int CMSFract::OnAccess(CPoint *pp)
{
	CMenu *pMenu = GetPopupMenu();
	CString str = GetRuntimeClass()->m_lpszClassName;
	pMenu->AppendMenu(MF_STRING, 0, str+": "+GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	pMenu->AppendMenu(MF_STRING, CMD_CHANGE_TO_UNKNOWN,"&unknown");
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

///////////////////// CMSRealEx /////////////////////
///////////////////// CMSRealEx /////////////////////

void CMSRealEx::CopyBody(CViecFC *pelm)
{
	CMSRealEx *pCp = (CMSRealEx *) pelm;
	CMSNumber::CopyBody(pelm);
	pCp->m_value = m_value;
}

void CMSRealEx::SerializeBody(CArchive& ar)
{
//	CMSNumber::SerializeBody(ar);
	if(ar.IsStoring()) {
		ar << m_value;
	} else {
		ar >> m_value;
	}
}

int CMSRealEx::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	CDC *pdc = GetDC();
	CString str;
	if(pdc==NULL) return FALSE;
	str.Format("%lf", m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(str, lstrlen(str));
	ReleaseDC(pdc);
	m_extent.left = 0;
	m_extent.right = sz.cx;
	m_extent.top = - sz.cy/2;
	m_extent.bottom = sz.cy + m_extent.top;
	if(dir==dirUp)
		GetParent()->CalcExt(dir);
	if(ext!=m_extent) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CMSRealEx::OnDraw(CDC *pdc, const CPoint& p)
{
	CString str;
	str.Format("%lf", m_value);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(str, lstrlen(str));
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);
	CPoint p0 = p;
	p0.Offset(GetOrg());
	pdc->TextOut(p0.x, p0.y-sz.cy/2, str);
}

int CMSRealEx::OnAccess(CPoint *pp)
{
	CMenu *pMenu = GetPopupMenu(TRUE);
	CString str;
	str.Format("%s: %lf", GetRuntimeClass()->m_lpszClassName, m_value);
	pMenu->AppendMenu(MF_STRING, 0, str);
	pMenu->AppendMenu(MF_SEPARATOR);
	CMenu *psub = GetPopupMenu(FALSE);
	pMenu->AppendMenu(MF_STRING, CMD_INPUT_VALUE, "&input value");
	if(GetParent()->SendMsg(MSG_CAN_THIS_DISAPPEAR, this))
		pMenu->AppendMenu(MF_STRING, CMD_ADD_DELETE, "&Delete");
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

int CMSRealEx::OnCommand(UINT id)
{
	switch(id) {
	case CMD_INPUT_VALUE:
		{
			CRealValueDlg dlg;
			dlg.m_value = m_value;
			dlg.m_label = "Real Number";
			if(dlg.DoModal()==IDOK) {
				m_value = dlg.m_value;
				m_extent.right++;
				GetParent()->SendMsg(MSG_CONST_CHANGED, this);
				CalcExt();
			}
		}
		return TRUE;
	}
	return CMSNumber::OnCommand(id);
}

/////////////////  CMSPDEMtx  //////////////////
/////////////////  CMSPDEMtx  //////////////////

int CMSPDEMtx::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	CViecFC *pI = GetChild(0);
	CViecFC *pJ = GetChild(1);

	CDC *pdc = GetDC();
	if(pI==NULL || pJ==NULL || pdc==NULL) return 0;
	if(dir==dirDown) {
		pI->CalcExt(dir);
		pJ->CalcExt(dir);
	}
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect xNm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect xBo = GetTextExtent(pdc, "[", 1);
	CRect xBc = GetTextExtent(pdc, "]", 1);
	CRect xCm = GetTextExtent(pdc, ",", 1);
	ReleaseDC(pdc);

	int nI = xNm.right + xBo.right;
	int nJ = nI + pI->GetExtent().Width() + xCm.right;
	pI->SetOrg(CPoint(nI, 0));
	pJ->SetOrg(CPoint(nJ, 0));
	m_extent = CRect(0,xNm.top, nJ + pJ->GetExtent().Width() + xBc.right, xNm.bottom);
	CalcExtPost(dir, ext);
	return TRUE;
}

void CMSPDEMtx::OnDraw(CDC *pdc, const CPoint& p)
{
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect xNm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect xBo = GetTextExtent(pdc, "[", 1);
	CRect xBc = GetTextExtent(pdc, "]", 1);
	CViecFC *pI = GetChild(0);

	CPoint p0 = GetOrg();
	p0.Offset(p);
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);

	TextOut(pdc, p0.x, p0.y, GetName(), GetNameLen());
	if(pI==NULL) return;

	int nBo = xNm.right;
	int nCm = nBo + xBo.right + pI->GetExtent().Width();
	int nBc = m_extent.right - xBc.right;

	TextOut(pdc, p0.x+nBo, p0.y, "[", 1);
	TextOut(pdc, p0.x+nCm, p0.y, ",", 1);
	TextOut(pdc, p0.x+nBc, p0.y, "]", 1);
	OnDrawChild(pdc, p);
}

/////////////////  CMSPDEVct //////////////////
/////////////////  CMSPDEVct //////////////////

int CMSPDEVct::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	CViecFC *pI = GetChild(0);

	CDC *pdc = GetDC();
	if(pI==NULL || pdc==NULL) return 0;
	if(dir==dirDown)
		pI->CalcExt(dir);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect xNm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect xBo = GetTextExtent(pdc, "[", 1);
	CRect xBc = GetTextExtent(pdc, "]", 1);
	ReleaseDC(pdc);

	int nI = xNm.right + xBo.right;
	pI->SetOrg(CPoint(nI, 0));
	m_extent = CRect(0,xNm.top, nI + pI->GetExtent().Width() + xBc.right, xNm.bottom);
	CalcExtPost(dir, ext);
	return TRUE;
}

void CMSPDEVct::OnDraw(CDC *pdc, const CPoint& p)
{
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect xNm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect xBo = GetTextExtent(pdc, "[", 1);
	CRect xBc = GetTextExtent(pdc, "]", 1);

	CPoint p0 = GetOrg();
	p0.Offset(p);
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);

	TextOut(pdc, p0.x, p0.y, GetName(), GetNameLen());

	int nBo = xNm.right;
	int nBc = m_extent.right - xBc.right;

	TextOut(pdc, p0.x+nBo, p0.y, "[", 1);
	TextOut(pdc, p0.x+nBc, p0.y, "]", 1);
	OnDrawChild(pdc, p);
}

////////////////////// CEFile //////////////////////////
////////////////////// CEFile //////////////////////////

int CEFile::CalcExt(dirMode dir)
{
	CRect extent = m_extent;
	m_extent = CRect(0,0,0,0);
	CDC *pdc = GetDC();
	if(!pdc) return FALSE;
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	ReleaseDC(pdc);
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	UnionExtent(m_extent, cir);
	UnionExtent(m_extent, nm);
	CalcExtPost(dir, extent);
	return TRUE;
}

void CEFile::OnDraw(CDC *pdc, const CPoint& p)
{
	CPoint p0 = p;
	p0.Offset(GetOrg());
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	EtchingRect(cir, CRect(2,2,-2,-2));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	
	SelectStockObject(pdc, BLACK_PEN);
	COLORREF col = m_bFocus	? RGB(0,180,0) : RGB(0,0,0);
	SetBrush(pdc, col);
	cir.OffsetRect(p0);
	pdc->Ellipse(&cir);
	SetTextColor(pdc, col);
	TextOut(pdc, p0.x + nm.left, p0.y, GetName(), GetNameLen());
}

void CEFile::ToDraw(CSymbInst *pInst, CDC *pdc, const CPoint& shift)
{
	CPoint p0 = shift;
	p0.Offset(-pInst->GetOrg());

	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	EtchingRect(cir, CRect(2,2,-2,-2));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	
	SelectStockObject(pdc, BLACK_PEN);
	COLORREF col = pInst->m_bFocus	? RGB(0,180,0) : RGB(0,0,0);
	SetBrush(pdc, col);
	cir.OffsetRect(p0);
	pdc->Ellipse(&cir);
	SetTextColor(pdc, col);
	TextOut(pdc, shift.x + nm.left, shift.y, GetName(), GetNameLen());
}

CRect CEFile::ToCalcExt(CSymbInst *ps, dirMode dir)
{
	CRect rt(0,0,0,0);
	CDC *pdc = GetDC();
	if(!pdc) return FALSE;
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	ReleaseDC(pdc);
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	UnionExtent(rt, cir);
	UnionExtent(rt, nm);
	ps->SetExtent(rt);
	return rt;
}

///////////////// CEFileFrom ////////////////
///////////////// CEFileFrom ////////////////
int CEFileFrom::CalcExt(dirMode dir)
{
	CRect extent = m_extent;
	m_extent = CRect(0,0,0,0);
	CDC *pdc = GetDC();
	if(!pdc) return FALSE;
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	ReleaseDC(pdc);
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	UnionExtent(m_extent, cir);
	UnionExtent(m_extent, nm);
	CalcExtPost(dir, extent);
	return TRUE;
}

void CEFileFrom::OnDraw(CDC *pdc, const CPoint& p)
{
	CPoint p0 = p;
	p0.Offset(GetOrg());
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	EtchingRect(cir, CRect(2,2,-2,-2));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	
	SelectStockObject(pdc, BLACK_PEN);
	COLORREF col = m_bFocus	? RGB(0,180,0) : RGB(0,0,0);
	SetBrush(pdc, col);
	cir.OffsetRect(p0);
	pdc->Ellipse(&cir);
	SetTextColor(pdc, col);
	TextOut(pdc, p0.x + nm.left, p0.y, GetName(), GetNameLen());
}

void CEFileFrom::ToDraw(CSymbInst *pInst, CDC *pdc, const CPoint& shift)
{
	CPoint p0 = shift;
	p0.Offset(-pInst->GetOrg());

	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	EtchingRect(cir, CRect(2,2,-2,-2));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	
	SelectStockObject(pdc, BLACK_PEN);
	COLORREF col = pInst->m_bFocus	? RGB(0,180,0) : RGB(0,0,0);
	SetBrush(pdc, col);
	cir.OffsetRect(p0);
	pdc->Ellipse(&cir);
	SetTextColor(pdc, col);
	TextOut(pdc, shift.x + nm.left, shift.y, GetName(), GetNameLen());
}

CRect CEFileFrom::ToCalcExt(CSymbInst *ps, dirMode dir)
{
	CRect rt(0,0,0,0);
	CDC *pdc = GetDC();
	if(!pdc) return FALSE;
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	ReleaseDC(pdc);
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	UnionExtent(rt, cir);
	UnionExtent(rt, nm);
	ps->SetExtent(rt);
	return rt;
}

///////////////// CPDEBttn //////////////////
///////////////// CPDEBttn //////////////////

int CPDEBttn::CalcExt(dirMode dir)
{
	if(m_extent.Width()==0) {
		m_color = RGB(255,255,255);
		m_extent = CRect(0,-5, 10,5);
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CPDEBttn::OnDraw(CDC *pdc, const CPoint& p)
{
	CPoint p0 = GetOrg();
	p0.Offset(p);
	CRect rct = GetExtent();
	rct.OffsetRect(p0);
	SetPen(pdc, PS_SOLID, 1, RGB(0,0,0));
	SetBrush(pdc, m_color);
	pdc->Rectangle(&rct);
}

int CPDEBttn::OnDrop(const CPoint& p)
{
	if(m_bFocus) {
		GetParent()->SendMsg(MSG_BTTN_PRESSED, this);
		InvalidateRect(CPoint(0,0));
	}
	return 0;
}

int CPDEBttn::OnFocus(CPoint &p)
{
	CRect r = m_extent;
	r.OffsetRect(m_org);
	m_bFocus = r.PtInRect(p)? TRUE:FALSE;
	if(m_bFocus) {
		InvalidateRect(CPoint(0,0));
	}
	return m_bFocus;
}



