
#include "stdafx.h"
#include <math.h>

#include "pde_prg.h"
#include "jd_prg.h"

#include "pde_msg.h"
#include "pad_msg.h"
#include "var_wtc.h"

#include "msg_adv.h"
#include "msg_prg.h"

IMPLEMENT_ELEM(CPDEProblem, CPDEBase, 223)
IMPLEMENT_ELEM(CPDEObjPrg, CPDEObject, 262)
IMPLEMENT_ELEM(CPDBrkPnt, CPDSubst, 267)
IMPLEMENT_ELEM(CPDFile, CPad, 276)
IMPLEMENT_ELEM(CPDEMtxDefs, CPDEBase, 236)
IMPLEMENT_ELEM(CPDEBrkPntDefs, CPDEMtxDefs, 237)
IMPLEMENT_ELEM(CPDEBrkObj, CPDEBase, 264)
IMPLEMENT_ELEM(CPDEConstDef, CMSIndex, 240)
IMPLEMENT_ELEM(CPDESngVarBox, CPDEBase, 300)
IMPLEMENT_ELEM(CPDESngVar, CViecFC, 301)
IMPLEMENT_ELEM(CPDFileFrom, CPad, 276)

/////////////////  CPDEProblem  /////////////////////
/////////////////  CPDEProblem  /////////////////////

CPDEProblem::CPDEProblem()
{
	m_pdeInfo = NULL;
	m_bDragAllow = TRUE;
}

int CPDEProblem::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	if(GetCount()==0) m_extent = CRect(0,0,100,70);
	else {
		CalcExtMidVert(dir, 20);
		CViecFC::CalcExt(dirNot);
	}

	m_extent.left -= 8;
	m_extent.right += 8;
	m_extent.top -= 6;
	m_extent.bottom += 6;
	if(dir==dirUp)
		GetParent()->CalcExt(dir);
	if(ext!=m_extent) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CPDEProblem::OnDraw(CDC *pdc, const CPoint& p)
{
	DrawFold3D(pdc, p, m_bFocus? 0:1);
	OnDrawChild(pdc, p);
}

BOOL CPDEProblem::OnCreated(EnumOnCreated bSkp)
{
	POSITION pos;
	CViecFC *pelm;
	pos = GetHeadPosition();
	while(pos) {
		pelm = GetNext(pos);
		if(pelm->IsKindOf(RUNTIME_CLASS(CPDEInfo))) {
			m_pdeInfo = (CPDEInfo *) pelm;
			break;
		}
	}
	if(m_pdeInfo && m_pdeInfo->IsKindOf(RUNTIME_CLASS(CIElem)))
		m_pdeInfo = (CPDEInfo *) ((CIElem *)m_pdeInfo)->GetInstBody();
	return CPDEBase::OnCreated(bSkp);
}

int CPDEProblem::OnAccess(CPoint *pp)
{
	CMenu *pMenu = GetPopupMenu(TRUE);
	CString str = GetRuntimeClass()->m_lpszClassName;
	pMenu->AppendMenu(MF_STRING, 0, str+": "+GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	CMenu *psub = GetPopupMenu(FALSE);
	pMenu->AppendMenu(MF_POPUP, (UINT) psub->m_hMenu, "&default menu");
	pMenu->AppendMenu(MF_SEPARATOR);
	AppendDefaultMenu(psub);

	if(GetCount()<=2) {
		pMenu->AppendMenu(MF_STRING, CMD_SET_1D_INFO, "&1 dim");
		pMenu->AppendMenu(MF_STRING, CMD_SET_2D_INFO, "&2 dim");
		pMenu->AppendMenu(MF_STRING, CMD_SET_3D_INFO, "&3 dim");
		pMenu->AppendMenu(MF_STRING, CMD_LOAD_PDE_INFO, "&load info");
		pMenu->AppendMenu(MF_STRING, CMD_SUBST_PROBLEM, "&substitue problem");
		pMenu->AppendMenu(MF_SEPARATOR);
	}
	if(GetCount()>0) {
		pMenu->AppendMenu(MF_STRING, CMD_PDE_RUN_SPREAD, "&run");  // For program design
	}
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

int CPDEProblem::OnCommand(UINT id)
{
	switch(id) {
	case CMD_LOAD_PDE_PROB_INFO:
		{
		CArchive *ar;
		CString head;
		ar = OpenFile("..\\temp\\_com.tmp", modeRead);
		if(ar==NULL) break;
		*ar >> head;
		CViecFC *pelm, *pOld;
		pOld = NULL;
		pelm = GetUnknownElem(*ar, head);
		CloseFile();
		if(pelm==NULL) return FALSE;

		if(GetCount()==0) {
			AddTail(pelm);
		} else {
			pOld = GetChild(0);
			SubstChild(pOld, pelm);
		}
		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pelm->CalcExt(dirDown);
		pelm->SendMsg(MSG_RECALC_ALL_VIEW, NULL);
		CalcExt();
		pelm->InvalidateRect(CPoint(0,0));
		if(pOld)
			delete pOld;
		return TRUE;
		}
	case CMD_LOAD_PDE_MESH_INFO:
		{
		CArchive *ar;
		CString head;
		CViecFC *pInf;
		if(GetCount()==0) return TRUE;
		pInf = GetChild(0);
		
		ar = OpenFile("..\\temp\\_com.tmp", modeRead);
		if(ar==NULL) break;
		*ar >> head;
		CViecFC *pelm, *pOld;
		pOld = NULL;
		pelm = GetUnknownElem(*ar, head);
		CloseFile();
		if(pelm==NULL) return FALSE;

		if(pInf->GetCount()==0) {
			pInf->AddTail(pelm);
		} else {
			pOld = pInf->GetChild(0);
			pInf->SubstChild(pOld, pelm);
		}
		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pelm->CalcExt(dirDown);
		pelm->SendMsg(MSG_RECALC_ALL_VIEW,NULL);
		//pInf->SendCmdQ(CMD_RECAL_SIZE);
		CalcExt();
		pelm->InvalidateRect(CPoint(0,0));
		if(pOld)
			delete pOld;
		return TRUE;
		}
	case CMD_LOAD_PDE_TIME_INFO:
		{
		CArchive *ar;
		CString head;
		CViecFC *pInf;
		if(GetCount()==0) return TRUE;
		pInf = GetChild(0);
		if(pInf->GetCount()==0) return TRUE;

		ar = OpenFile("..\\temp\\_com.tmp", modeRead);
		if(ar==NULL) break;
		*ar >> head;
		CViecFC *pelm, *pOld;
		pOld = NULL;
		pelm = GetUnknownElem(*ar, head);
		CloseFile();
		if(pelm==NULL) return FALSE;

		if(pInf->GetCount()==1) {
			pInf->AddTail(pelm);
		} else {
			pOld = pInf->GetChild(1);
			pInf->SubstChild(pOld, pelm);
		}
		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pelm->CalcExt(dirDown);
		pelm->SendMsg(MSG_RECALC_ALL_VIEW, NULL);
		//pInf->SendCmdQ(CMD_RECAL_SIZE);
		CalcExt();
		pelm->InvalidateRect(CPoint(0,0));
		if(pOld)
			delete pOld;
		return TRUE;
		}
	case CMD_LOAD_PDE_DEP_INFO:
		{
		CArchive *ar;
		CString head;
		CViecFC *pInf;
		if(GetCount()==0) return TRUE;
		pInf = GetChild(0);
		if(pInf->GetCount()<2) return TRUE;

		ar = OpenFile("..\\temp\\_com.tmp", modeRead); 
		//ar = OpenFile("..\\temp\\depinfo.obj", modeRead); //RgĂB
		if(ar==NULL) break;
		*ar >> head;
		CViecFC *pelm, *pOld;
		pOld = NULL;
		pelm = GetUnknownElem(*ar, head);
		CloseFile();
		if(pelm==NULL) return FALSE;

		pInf->AddTail(pelm);
		/*
		if(pInf->GetCount()==2) {
			pInf->AddTail(pelm);
		} else {
			pOld = pInf->GetChild(2);
			pInf->SubstChild(pOld, pelm);
		}
		*/
		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pelm->CalcExt(dirDown);
		pelm->SendMsg(MSG_RECALC_ALL_VIEW, NULL);
		pInf->SendCmdQ(CMD_RECAL_SIZE);
		CalcExt();
		pelm->InvalidateRect(CPoint(0,0));
		if(pOld)
			delete pOld;
		return TRUE;
		}
	case CMD_LOAD_PDE_DEP_DEL:
		{
		CViecFC *pInf;
		if(GetCount()==0) return TRUE;
		pInf = GetChild(0);
		if(pInf->GetCount()<2) return TRUE;

		CViecFC *pOld;
		pOld = NULL;
		
		POSITION pos;
		//BOOL DEPFlg;
		//ParaFlg = FALSE; //CPDEDepInfo݂邩AȂ
		
		//CPDEInfo̎q𒲂ׂ
		pos = pInf->GetHeadPosition();
		while(pos){
			pOld = (CViecFC*) pInf->GetNext(pos);
			if(pOld->IsKindOf(RUNTIME_CLASS(CPDEDepInfo))){ 
				pOld->SendCmdQ(CMD_ADD_DELETE);
				//ParaFlg = TRUE;
			}
		}

		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pInf->SendCmdQ(CMD_RECAL_SIZE);
		CalcExt();
		if(pOld)
			pOld = NULL;
		return TRUE;
		}
	/*
	case CMD_LOAD_PDE_SLV_SET:
		{
		CArchive *ar;
		CString head;
		CViecFC *pInf;
		if(GetCount()==0) return TRUE;
		pInf = GetChild(0);
		if(pInf->GetCount()<2) return TRUE;

		ar = OpenFile("..\\temp\\T_solv.obj", modeRead);
		if(ar==NULL) break;
		*ar >> head;
		CViecFC *pelm, *pOld, *pOld2;
		pOld = NULL;
		pelm = GetUnknownElem(*ar, head);
		CloseFile();
		if(pelm==NULL) return FALSE;

		POSITION pos,pos2;
		BOOL SlvFlg;
		SlvFlg = FALSE; //CPDESolvset݂邩AȂ
		
		//CPDEInfo̎q𒲂ׂ
		pos = pInf->GetHeadPosition();
		while(pos){
			pOld = (CViecFC*) pInf->GetNext(pos);
			if(pOld->IsKindOf(RUNTIME_CLASS(CPDEDepInfo))){ 
				pos2 = pOld->GetHeadPosition();
				while(pos2){ //CPDEDepInfo̎q𒲂ׂ
					pOld2 = (CViecFC*) pOld->GetNext(pos2);
					if(pOld2->IsKindOf(RUNTIME_CLASS(CPDESolvSet))){ 
						SlvFlg = TRUE;
						break;
					}
				}
				if(SlvFlg == TRUE){
					SlvFlg = FALSE;
					continue;
				}
				else {
					pOld->AddTail(pelm);
					SlvFlg = FALSE;
					break;
				}
			}
		}
		
		//if(pInf->GetCount()==2) {
		//	pInf->AddTail(pelm);
		//} else {
		//	pOld = pInf->GetChild(2);
		//	pInf->SubstChild(pOld, pelm);
		//}
		
		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pelm->CalcExt(dirDown);
		pelm->SendMsg(MSG_RECALC_ALL_VIEW, NULL);
		pOld->SendCmdQ(CMD_RECAL_SIZE);
		CalcExt();
		pelm->InvalidateRect(CPoint(0,0));
		
		//if(pOld)
		//	delete pOld;
		//
		return TRUE;
		}
	*/
	case CMD_LOAD_PDE_PARA_INFO:
		{
		CArchive *ar;
		CString head;
		CViecFC *pInf;
		if(GetCount()==0) return TRUE;
		pInf = GetChild(0);
		if(pInf->GetCount()<2) return TRUE;

		ar = OpenFile("..\\temp\\_com.tmp", modeRead);
		// ar = OpenFile("..\\temp\\parainfo.obj", modeRead);  fobNpt@Ĉnodm
		if(ar==NULL) break;
		*ar >> head;
		CViecFC *pelm, *pOld;
		pOld = NULL;
		pelm = GetUnknownElem(*ar, head);
		CloseFile();
		if(pelm==NULL) return FALSE;

		POSITION pos;
		BOOL ParaFlg;
		ParaFlg = FALSE; //CPDEParaInfo݂邩AȂ
		
		//CPDEInfo̎q𒲂ׂ
		pos = pInf->GetHeadPosition();
		while(pos){
			pOld = (CViecFC*) pInf->GetNext(pos);
			if(pOld->IsKindOf(RUNTIME_CLASS(CPDEParaInfo))){ 
				pInf->SubstChild(pOld, pelm); //pOld̈ʒupelmaddtailB
				ParaFlg = TRUE;
			}
		}
		if(ParaFlg == FALSE) {
			pInf->AddTail(pelm);
		}
		
		s_MathSymbMapTable.RemoveAll();
		OnCreated(DO_ALL);
		pelm->CalcExt(dirDown);
		pelm->SendMsg(MSG_RECALC_ALL_VIEW, NULL);
		pInf->SendCmdQ(CMD_RECAL_SIZE);
		CalcExt();
		pelm->InvalidateRect(CPoint(0,0));
		if(pOld)
			//delete pOld;
			pOld = NULL;
		return TRUE;
		}
	case CMD_PDE_RUN_SPREAD:   // For program design
		{
			CPDERunSpread *pRun;
			POSITION pos, pos2;
			CPDEDepInfo *pDep;
			CPDESolvSet *pSet;
			CString head;
			CViecFC *pelm;

			pRun = new CPDERunSpread;
			SetMode(modeOpen);
			              // mpdeInfo:  
			pos = m_pdeInfo->GetHeadPosition();	 // get head position directly connected child
			// pos: head pointer of list, each node has a pointer & link pointer 
			// link pointer points the real objetc of the child.
			while(pos) {
				pDep = (CPDEDepInfo *) m_pdeInfo->GetNext(pos);
				if(pDep->IsKindOf(RUNTIME_CLASS(CIElem)))	
					pDep = (CPDEDepInfo *) ((CIElem*)pDep)->GetBody();
 				           //if it is an object splitted
                           // from its body.
				           // "I" of C"I"Elem means   its is splitted from its body
				           // GetBody:  search the body
				if(!pDep->IsKindOf(RUNTIME_CLASS(CPDEDepInfo)))
					continue;	  // check if it is CPDEDepInfo
				pos2 = pDep->GetHeadPosition();
				while(pos2) {
					pSet = (CPDESolvSet *) pDep->GetNext(pos2);
					if(pSet->IsKindOf(RUNTIME_CLASS(CIElem)))	  //check if it is splitted
						pSet = (CPDESolvSet *) ((CIElem*) pSet)->GetBody();
					if(!pSet->IsKindOf(RUNTIME_CLASS(CPDESolvSet)))
						continue;				   // find Solverset
					pelm = (CViecFC *) pSet->Copy(); // copy the solverset object except its name
					pelm->SetName(pSet->GetName(), m_nDot, ANSI_CHARSET); //set the name to its original name
					pRun->CreateNewTemplate(pelm);  // doing other jobs.
					  // creating 3 containers, realizing the Compute-Store Model
					  //  (ObjC: Inut)-> (ProcD: ProcessDescripter)-> (ObjC: output)
					  // After &run command, you see 1 process containing 3 subproceses
					  //  The 3 boxes for 3 subprocesses are realized.  The contents are NULL. 
				}
			}
			pRun->SetName(m_pdeInfo->GetName()+CString(".Run"), m_nDot, m_nCharSet);
			AddTail(pRun);	   //Add .Run to its name (see running GUI). (ObjC)

			CPDEObjPrg *pPrg = new CPDEObjPrg;	   // for program container (ProcD) 
			AddTail(pPrg);
			pPrg->SetName(m_pdeInfo->GetName()+CString(".Prog"), m_nDot, m_nCharSet);

			pRun->AddInputPoint(m_pdeInfo);	  // add input with arrow; from m_pdeInfo
			pRun->AddOutputPoint(pPrg);		  // add output with arrow; to pPrg

			pPrg->OnCreated(); pRun->OnCreated();//Kawa
			//OnCreated();//Kawa
			                // at this point PDEBprb has 3 children 
			                //maybe better to do pPrg->OnCreated(); pRun->OnCreated();
			CalcExt(dirDown);	 //PDEPrb->CalcExt;  dirDown: to downward
			pRun->SendMsg(MSG_RECALC_ALL_VIEW, NULL); // re-calcExt & InvalidateRect for safe.
			CalcExt();		// again for safe.
		}
		return TRUE;
	case CMD_SET_1D_INFO:										 
		{
			while(GetCount())
				delete m_child.RemoveTail();
			CPDEInfo *pPdeInfo = new CPDEInfo;

			CPDEMeshInfo *pMshInfo = new CPDEMeshInfo;
			CPDETimeVar *pTmV = new CPDETimeVar;
			pPdeInfo->AddTail(pMshInfo);
			pPdeInfo->AddTail(pTmV);

			pTmV->SetName("Time Var");
			pTmV->CreateTimeVar("t", "dt", "n", "NMAX", 1000);
			pTmV->SetMode(modeClose);

			pMshInfo->SetName("Mesh Info.");
			pMshInfo->CreateMeshInfo1D();
			pMshInfo->SetMode(modeClose);

			AddTail(pPdeInfo);
			OnCreated();	// Initialize
			CalcExt(dirDown);	// size calculation
			CalcExt();
//			pPdeInfo->ToDblBody();
		}
		return TRUE;
	case CMD_SET_2D_INFO:
		{
			while(GetCount())
				delete m_child.RemoveTail();
			CPDEInfo *pPdeInfo = new CPDEInfo;
			CPDEMeshInfo *pMshInfo = new CPDEMeshInfo;
			CPDETimeVar *pTmV = new CPDETimeVar;
			pPdeInfo->AddTail(pMshInfo);
			pPdeInfo->AddTail(pTmV);

			pTmV->SetName("Time Var");
			pTmV->CreateTimeVar("t", "dt", "n", "NMAX", 1000);
			pTmV->SetMode(modeClose);

			pMshInfo->SetName("Mesh Info.");
			pMshInfo->CreateMeshInfo2();
			pMshInfo->SetMode(modeClose);

			AddTail(pPdeInfo);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
//			pPdeInfo->ToDblBody();
		}
		return TRUE;
		case CMD_SET_3D_INFO:
		{
			while(GetCount())
				delete m_child.RemoveTail();
			CPDEInfo *pPdeInfo = new CPDEInfo;
			CPDEMeshInfo *pMshInfo = new CPDEMeshInfo;
			CPDETimeVar *pTmV = new CPDETimeVar;
			pPdeInfo->AddTail(pMshInfo);
			pPdeInfo->AddTail(pTmV);

			pTmV->SetName("Time Var");
			pTmV->CreateTimeVar("t", "dt", "n", "NMAX", 1000);
			pTmV->SetMode(modeClose);

			pMshInfo->SetName("Mesh Info.");
			pMshInfo->CreateMeshInfo3D();
			pMshInfo->SetMode(modeClose);

			AddTail(pPdeInfo);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
//			pPdeInfo->ToDblBody();
		}
		return TRUE;
	case CMD_LOAD_PDE_INFO:
		{
			CString head;
			CPDEInfo *pPdeInfo;
			CFileDialog dlgFile(TRUE, NULL, "*.*", OFN_FILEMUSTEXIST);
			if(dlgFile.DoModal()!=IDOK) break;
			OpenFile(dlgFile.GetPathName(), modeRead);
			if(m_pArc==NULL) break;
			*m_pArc >> head;
			if(head!=CString("CPDEInfo")) {
				MessageBox(NULL, "This is not proper one.", "ERR", MB_OK);
				return TRUE;
			}
			pPdeInfo = (CPDEInfo *) GetUnknownElem(*m_pArc, head);
			CloseFile();
			if(pPdeInfo==NULL) return TRUE;
			if(!pPdeInfo->IsKindOf(RUNTIME_CLASS(CPDEInfo))) {
				MessageBox(NULL, "This is not proper one #.", "ERR", MB_OK);
				return TRUE;
			}
			while(GetCount())
				delete m_child.RemoveTail();
			AddTail(pPdeInfo);
			OnCreated();
			CalcExt(dirDown);
			SendMsg(MSG_RECALC_ALL_VIEW, NULL);
			CalcExt();
//			pPdeInfo->ToDblBody();
			return TRUE;
		}
		return TRUE;
	case CMD_SUBST_PROBLEM:
		{
			CString head;
			CPDEProblem *pPdeProb;
			POSITION pos;
			CFileDialog dlgFile(TRUE, NULL, "*.prb", OFN_FILEMUSTEXIST);
			if(dlgFile.DoModal()!=IDOK) break;
			OpenFile(dlgFile.GetPathName(), modeRead);
			if(m_pArc==NULL) break;
			*m_pArc >> head;
			if(head!=CString("CPDEProblem")) {
				MessageBox(NULL, "This is not proper one.", "ERR", MB_OK);
				return TRUE;
			}
			pPdeProb = (CPDEProblem *) GetUnknownElem(*m_pArc, head);
			CloseFile();
			if(pPdeProb==NULL) return TRUE;
			if(!pPdeProb->IsKindOf(RUNTIME_CLASS(CPDEProblem))) {
				MessageBox(NULL, "This is not proper one #.", "ERR", MB_OK);
				return TRUE;
			}
			while(GetCount())
				delete m_child.RemoveTail();
			pos = pPdeProb->GetHeadPosition();
			while(pos) {
				AddTail(pPdeProb->GetNext(pos));
			}
			pPdeProb->m_child.RemoveAll();
			delete pPdeProb;
			OnCreated();
//			CalcExt(dirDown);
			SendMsg(MSG_RECALC_ALL_VIEW, NULL);
//			CalcExt();
		}
		return TRUE;
	}
	return CPDEBase::OnCommand(id);
}

BOOL CPDEProblem::SendMsg(int msg, CObject* el)
{
	switch(msg) {
	case MSG_CAN_THIS_DISAPPEAR:
		if(el==m_pdeInfo)
			return FALSE;
		return TRUE;
	}
	return CPDEBase::SendMsg(msg, el);
}

/////////////////  CPDEObjPrg  //////////////////
/////////////////  CPDEObjPrg  //////////////////
/////////////////  CPDEObjPrg  //////////////////

CPDEObjPrg::CPDEObjPrg()
{
	m_bGen = FALSE;
	m_bBreaking = FALSE;
	hVar = NULL;
	hSig = NULL;
	hThr = NULL;
	pVar = NULL;
	m_pBrkPnt = NULL;
	m_pBrkObj = NULL;

	startupInfo.cb = sizeof(STARTUPINFO);
	startupInfo.lpReserved = NULL;
	startupInfo.lpDesktop = NULL;
	startupInfo.lpTitle = NULL;
	startupInfo.dwX = 0;
	startupInfo.dwY = 0;
	startupInfo.dwXSize = 0;
	startupInfo.dwYSize = 0;
	startupInfo.dwXCountChars = 0;
	startupInfo.dwYCountChars = 0;
	startupInfo.dwFillAttribute = NULL;
	startupInfo.dwFlags = 0;
	startupInfo.wShowWindow = 0;
	startupInfo.cbReserved2 = 0;
	startupInfo.lpReserved2 = NULL;
	startupInfo.hStdInput = NULL;
	startupInfo.hStdOutput = NULL;
	startupInfo.hStdError = NULL;

}

CPDEObjPrg::~CPDEObjPrg()
{
	DeleteCriticalSection(&memSync);
}

BOOL CPDEObjPrg::OnCreated(EnumOnCreated bSkp)
{
	InitializeCriticalSection(&memSync);
	return CPDEBase::OnCreated(bSkp);
}

void CPDEObjPrg::SerializeBody(CArchive& ar)
{
	BYTE b;
	CPDEObject::SerializeBody(ar);
	if(ar.IsStoring()) {
		ar << (BYTE) m_bGen;
		ar << sPathName;
		ar << sFileName;
		ar << sFileExt;
		ar << sFileTitle;
		ar << sPath;
	} else {
		ar >> b;
		m_bGen = b;
		ar >> sPathName;
		ar >> sFileName;
		ar >> sFileExt;
		ar >> sFileTitle;
		ar >> sPath;
	}
}

int CPDEObjPrg::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	if(GetCount()==0 || GetMode()!=modeOpen)
		return CPDEBase::CalcExt(dir);
	CalcExtLines(dir);

	m_extent.left -= 8;
	m_extent.right += 8;
	m_extent.top -= 6;
	m_extent.bottom += 6;
	if(dir==dirUp)
		GetParent()->CalcExt(dir);
	if(ext!=m_extent) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CPDEObjPrg::OnDraw(CDC *pdc, const CPoint& p)
{
	DrawFold3D(pdc, p, m_bFocus? 0:1);
	if(m_mode!=modeClose) {
		OnDrawChild(pdc, p);
	} else {
		CRect r0=m_extent;
		r0.OffsetRect(m_org);
		r0.OffsetRect(p);
		CRect rD(r0.left+4, r0.top+4, r0.right-4, r0.bottom-4);
		if(m_bFocus) pdc->SetTextColor(RGB(0,0,255));
		SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
		pdc->DrawText(GetName(), lstrlen(GetName()), &rD, GetFormat());
	}
}

int CPDEObjPrg::OnAccess(CPoint *pp)
{
	CMenu *pMenu = GetPopupMenu(TRUE);
	CString str = GetRuntimeClass()->m_lpszClassName;
	pMenu->AppendMenu(MF_STRING, 0, str+": "+GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	CMenu *psub = GetPopupMenu(FALSE);
	pMenu->AppendMenu(MF_POPUP, (UINT) psub->m_hMenu, "&default menu");
	pMenu->AppendMenu(MF_SEPARATOR);
	AppendDefaultMenu(psub);

	if(hThr) {
		pMenu->AppendMenu(MF_STRING, 0, "Now the program is running");
	} else {
		pMenu->AppendMenu(MF_STRING, CMD_RUN_CREATE_C_DEBUG, "create program in C de&bug version");
		if(m_bGen) {
			pMenu->AppendMenu(MF_STRING, CMD_COMPILE_RUN_THE_PROG, "&compile");
			pMenu->AppendMenu(MF_STRING, CMD_RUN_THE_PROG, "&run");
		}
	}
	pMenu->AppendMenu(MF_STRING, CMD_RUN_CREATE_C, "create program in &C");
	pMenu->AppendMenu(MF_STRING, CMD_ADD_3DIM_VAR_WATCH, "&3 dim var watcher");
	pMenu->AppendMenu(MF_STRING, CMD_ADD_2DIM_VAR_WATCH, "&2 dim var watcher");
	pMenu->AppendMenu(MF_STRING, CMD_ADD_1DIM_VAR_WATCH, "&1 dim var watcher");
	pMenu->AppendMenu(MF_STRING, CMD_ADD_SINGLE_VAR_BOX, "&single var box");

	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

DWORD WINAPI CompileAndRun(LPVOID pv)
{
	char buf[100];
	BOOL bi,bo;
	HANDLE hChildStdoutRd, hChildStdoutWr, hSaveStdout;
	SECURITY_ATTRIBUTES saAttr;
	DWORD dwRead;

	if(pv==NULL) return 0;
	CPDEObjPrg *pPrg = (CPDEObjPrg *) pv;
	if(!pPrg->IsKindOf(RUNTIME_CLASS(CPDEObjPrg))) return 0;

	dlgPrg.m_sLabel.Format("%s program is being compiled.", (const char *) pPrg->GetName());
	pPrg->SendCmdQ(CMD_SHOW_PROGRESS_DLG);

	pPrg->procInfo.hProcess = NULL;
	saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
	saAttr.bInheritHandle = TRUE;
	saAttr.lpSecurityDescriptor = NULL;
	hSaveStdout = GetStdHandle(STD_OUTPUT_HANDLE); 
	bi = CreatePipe(&hChildStdoutRd, &hChildStdoutWr, &saAttr, 0);
	pPrg->startupInfo.hStdOutput = hChildStdoutWr;
	//--- compiling
	sprintf(buf, "cl.exe %s\\%s.c", (const char *) pPrg->sPath, (const char *) pPrg->sFileName);
	pPrg->startupInfo.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
	if(!CreateProcess (NULL, buf,NULL,NULL,TRUE,0,NULL,NULL, &pPrg->startupInfo, &pPrg->procInfo)) {
		goto to_end;
	}
	bo = SetStdHandle(STD_OUTPUT_HANDLE, hSaveStdout);
	CloseHandle(hChildStdoutWr); hChildStdoutWr = NULL;
	for(;;) {
	if (!ReadFile(hChildStdoutRd, buf, 100, &dwRead, NULL) || dwRead == 0)
		break; 
	}
	CloseHandle(hChildStdoutRd); hChildStdoutRd = NULL;
	if(!strstr(buf, "\r\n-out:")) {
		goto to_end;
	}
to_end:
	pPrg->SendCmdQ(CMD_DESTROY_PROGRESS_DLG);

	pPrg->startupInfo.dwFlags = 0;
	pPrg->startupInfo.wShowWindow = 0;
	pPrg->procInfo.hProcess = NULL;
	if(hChildStdoutRd) CloseHandle(hChildStdoutRd);
	if(hChildStdoutWr) CloseHandle(hChildStdoutWr);
	pPrg->startupInfo.hStdOutput = NULL;
	pPrg->hThr = NULL;
	return 1;
}

CViecFCList BrkPntLst;

DWORD WINAPI WaitForSignal(LPVOID pv)
{
	char buf[30], *pchPnt, *pchBrk;
	CPDEBrkObj *pBrk;
	CPDBrkPnt *pPnt;
	POSITION pos;
	CSymbInst *pInst;

	if(pv==NULL) return 0;
	CPDEObjPrg *pPrg = (CPDEObjPrg *) pv;
	if(!pPrg->IsKindOf(RUNTIME_CLASS(CPDEObjPrg))) return 0;
	while(WaitForSingleObject(pPrg->hSig, INFINITE)==WAIT_OBJECT_0) {
		if(pPrg->pVar) {
			EnterCriticalSection(&pPrg->memSync);
			strcpy(buf, pPrg->pVar->pszStatus);
			LeaveCriticalSection(&pPrg->memSync);
			pPrg->m_bBreaking = TRUE;
			pchPnt = strtok(buf, ","); 
			pchBrk = strtok(NULL, ",");
			pos = BrkPntLst.GetHeadPosition();
			while(pos) {
				pPnt = (CPDBrkPnt *) BrkPntLst.GetNext(pos);
				if(!pPnt->IsKindOf(RUNTIME_CLASS(CPDBrkPnt))) continue;
				if(pPnt->GetCount()<1) continue;
				pInst = (CSymbInst *) pPnt->GetChild(0);
				if(!pInst || !pInst->IsKindOf(RUNTIME_CLASS(CSymbInst))) continue;
				pBrk = (CPDEBrkObj *) pInst->GetSymbBody();
				if(!pBrk || !pBrk->IsKindOf(RUNTIME_CLASS(CPDEBrkObj))) continue;
				if(strcmp(pPnt->GetName(), pchPnt)) continue;
				if(strcmp(pBrk->GetName(), pchBrk)) continue;
				pPrg->m_pBrkPnt = pPnt;
				pPrg->m_pBrkObj = pBrk;
				pPrg->SendCmdQ(MSG_SIGNAL_HAS_COME);
				if(!pBrk->m_bBreak && pBrk->hBrk) {
					SetEvent(pBrk->hBrk);
					pPrg->m_bBreaking = FALSE;
				}
			}
		}
	}
	return 1;
}

DWORD WINAPI RunWithoutCompile(LPVOID pv)
{
	char buf[100];
	DWORD tid;
	HANDLE hThr;

	if(pv==NULL) return 0;
	CPDEObjPrg *pPrg = (CPDEObjPrg *) pv;
	if(!pPrg->IsKindOf(RUNTIME_CLASS(CPDEObjPrg))) return 0;

	
	CSuitCase SC;
	BrkPntLst.RemoveAll();
	SC.m_runtimeClass = RUNTIME_CLASS(CPDBrkPnt);
	SC.m_oblist = &BrkPntLst;
	pPrg->SendMsg(MSG_COLLECT_OBJECT, &SC);

	pPrg->sSig.Format("%s_SIGNAL", (const char *) pPrg->GetName());
	pPrg->sVar.Format("%s_VARS", (const char *) pPrg->GetName());
	pPrg->sBrk.Format("%s_BRK", (const char *) pPrg->GetName());

	dlgPrg.m_sLabel.Format("%s program is running.", (const char *) pPrg->GetName());
	pPrg->SendCmdQ(CMD_SHOW_PROGRESS_DLG);

	pPrg->m_pBrkPnt = NULL;
	pPrg->m_pBrkObj = NULL;
	pPrg->procInfo.hProcess = NULL;
	pPrg->startupInfo.dwFlags = 0;
	pPrg->startupInfo.wShowWindow = 0;
	pPrg->hSig = CreateEvent(NULL, FALSE, FALSE, pPrg->sSig);
	sprintf(buf, "%s\\%s.exe", (const char *) pPrg->sPath, (const char *) pPrg->sFileName);
	if(!CreateProcess (NULL,buf,NULL,NULL,TRUE, 0,NULL, pPrg->sPath, &pPrg->startupInfo, &pPrg->procInfo))
		goto to_end;
	pPrg->InvalidateRect(CPoint(0,0));

	WaitForSingleObject(pPrg->hSig, INFINITE);

	pPrg->hVar = OpenFileMapping(FILE_MAP_WRITE, TRUE, pPrg->sVar);
	pPrg->pVar = (VAR_STRUCT *) MapViewOfFile(pPrg->hVar, FILE_MAP_WRITE, 0,0,0);
	pPrg->SendMsg(MSG_PROG_START_RUNNIG, NULL);

	hThr = ::CreateThread(NULL, 0, WaitForSignal, pv, 0, &tid);
	int i;
	pPrg->m_bBreaking = FALSE;
	for(i=1; WaitForSingleObject(pPrg->procInfo.hProcess, 1000)==WAIT_TIMEOUT; ) {
		int h = i/3600;
		int m = (i - h*3600)/60;
		int s = i - h*3600 - m*60;
		if(pPrg->m_bBreaking && pPrg->m_pBrkPnt && pPrg->m_pBrkObj) {
			dlgPrg.m_sLabel.Format("stop at %s(%s) %d sec (%02d:%02d:%02d)"
				, pPrg->m_pBrkPnt->GetName(), pPrg->m_pBrkObj->GetName(), i, h,m,s);
		} else {
			dlgPrg.m_sLabel.Format("running %d sec (%02d:%02d:%02d)", i, h,m,s);
			i++;
		}
		if(pPrg->SendCmdQ(CMD_IS_CANCELED_DLG)) {
			break;
		}
	}
	HANDLE hPrc;
	hPrc = OpenProcess(PROCESS_ALL_ACCESS, FALSE, pPrg->procInfo.dwProcessId);
	TerminateProcess(hPrc, TRUE);
	CloseHandle(pPrg->procInfo.hProcess);
	CloseHandle(hThr);

	dlgPrg.m_bCancel = FALSE;
	for(i=0;;i++) {
		int h = i/3600;
		int m = (i - h*3600)/60;
		int s = i - h*3600 - m*60;
		dlgPrg.m_sLabel.Format("stopped for %d sec (%02d:%02d:%02d)", i, h,m,s);
		if(pPrg->SendCmdQ(CMD_IS_CANCELED_DLG)) {
			break;
		}
		Sleep(1000);
	}
	pPrg->SendMsg(MSG_PROG_STOP_RUNNIG, NULL);
	EnterCriticalSection(&pPrg->memSync);
	CloseHandle(pPrg->hVar);
	pPrg->pVar = NULL;
	pPrg->hVar = NULL;
	LeaveCriticalSection(&pPrg->memSync);

to_end:
	pPrg->m_pBrkPnt = NULL;
	pPrg->m_pBrkObj = NULL;
	pPrg->m_bBreaking = FALSE;
	pPrg->SendCmdQ(CMD_DESTROY_PROGRESS_DLG);
	CloseHandle(pPrg->hSig);
	pPrg->pVar = NULL;
	pPrg->hVar = NULL;
	pPrg->hSig = NULL;
	pPrg->hThr = NULL;
	return 1;
}

int CPDEObjPrg::OnCommand(UINT id)
{
	DWORD tid;
	switch(id) {
	case MSG_SIGNAL_HAS_COME:
		SendMsg(MSG_SIGNAL_HAS_COME, m_pBrkPnt);
		return TRUE;
	case CMD_SHOW_PROGRESS_DLG:
		dlgPrg.m_bCancel = FALSE;
		dlgPrg.Create(CPrgrsDlg::IDD);
		dlgPrg.ShowWindow(SW_SHOW);
		return TRUE;
	case CMD_IS_CANCELED_DLG:
		if(dlgPrg.m_bCancel) return TRUE;
		dlgPrg.UpdateData(FALSE);
		return FALSE;
	case CMD_DESTROY_PROGRESS_DLG:
		dlgPrg.DestroyWindow();
		return TRUE;
	case CMD_ADD_3DIM_VAR_WATCH:
		{
			CPDE3DimVar *pVarW = new CPDE3DimVar;
			pVarW->Create();
			AddTail(pVarW);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
		}
		return TRUE;
	case CMD_ADD_2DIM_VAR_WATCH:
		{
			CPDE2DimVar *pVarW = new CPDE2DimVar;
			pVarW->Create();
			AddTail(pVarW);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
		}
		return TRUE;
	case CMD_ADD_1DIM_VAR_WATCH:
		{
			CPDE1DimVar *pVarW = new CPDE1DimVar;
			pVarW->Create();
			AddTail(pVarW);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
		}
		return TRUE;
	case CMD_ADD_SINGLE_VAR_BOX:
		{
			CPDESngVarBox *pBox = new CPDESngVarBox;
			AddTail(pBox);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
		}
		return TRUE;
	case CMD_RUN_CREATE_C:
		{
			CPDEProblem *pPrb;
			pPrb = (CPDEProblem *) GetParent();
			if(pPrb->IsKindOf(RUNTIME_CLASS(CElemView)))
				pPrb = (CPDEProblem *) GetInstBody()->GetParent();
			if(!pPrb->IsKindOf(RUNTIME_CLASS(CPDEProblem))) return TRUE;
			CFileDialog dlgFile(FALSE, NULL, "*.c", OFN_OVERWRITEPROMPT);
			if(dlgFile.DoModal()!=IDOK) break;

			CJDFract2Real real;			// code 98/7/27 
			SendMsg(MSG_DO_JOB, &real);

			CJDCreateProgInC progMaker;
			progMaker.m_bDebug = FALSE;
			progMaker.m_fileName = dlgFile.GetPathName();
			progMaker.m_mainProg = this;
			progMaker.m_pdeInfo = pPrb->m_pdeInfo;
			SendMsg(MSG_DO_JOB, &progMaker);
		}
		return TRUE;
	case CMD_RUN_CREATE_C_DEBUG:
		{
			CPDEProblem *pPrb;
			pPrb = (CPDEProblem *) GetParent();
			if(pPrb->IsKindOf(RUNTIME_CLASS(CElemView)))
				pPrb = (CPDEProblem *) GetInstBody()->GetParent();
			if(!pPrb->IsKindOf(RUNTIME_CLASS(CPDEProblem))) return TRUE;
			CFileDialog dlgFile(FALSE, NULL, "*.c", OFN_OVERWRITEPROMPT);
			if(dlgFile.DoModal()!=IDOK) break;
			sPathName = dlgFile.GetPathName();
			sFileName = dlgFile.GetFileName();
			if(sFileName.Right(2)==".c")
				sFileName = sFileName.Left(sFileName.GetLength()-2);
			sFileExt = dlgFile.GetFileExt();
			sFileTitle = dlgFile.GetFileTitle();
			sPath = sPathName.Left(sPathName.GetLength() - sFileName.GetLength() - 3);
			m_bGen = TRUE;

			CJDCreateProgInC progMaker;
			progMaker.m_bDebug = TRUE;
			progMaker.m_fileName = dlgFile.GetPathName();
			progMaker.m_mainProg = this;
			progMaker.m_pdeInfo = pPrb->m_pdeInfo;
			SendMsg(MSG_DO_JOB, &progMaker);
		}
		return TRUE;
	case CMD_COMPILE_RUN_THE_PROG:
		hThr = ::CreateThread(NULL, 0, CompileAndRun, this, 0, &tid);
		return TRUE;
	case CMD_RUN_THE_PROG:
		hThr = ::CreateThread(NULL, 0, RunWithoutCompile, this, 0, &tid);
		return TRUE;
	}
	return CPDEBase::OnCommand(id);
}

BOOL CPDEObjPrg::SetMenuForPDFor(CMenu *pMenu)  // No need for now
{
	CMenu *psub = GetPopupMenu(FALSE);
	pMenu->AppendMenu(MF_POPUP, (UINT)psub->m_hMenu, GetName());
	psub->AppendMenu(MF_STRING, CMD_NEW_FILE_INST, "Add &file inst");
	psub->AppendMenu(MF_STRING, CMD_PAD_NEW_BRK, "Add &break point");
	return TRUE;
}

void CPDEObjPrg::ToFindCandAndSelect(CViecFC *pE, CRuntimeClass *pRtc)
{
	int i;
	CSuitCase SC;
	CViecFCList obLst;

	strAry.RemoveAll();
	SC.m_oblist = &obLst;
	SC.m_runtimeClass = pRtc;
	GetAncestor()->SendMsg(MSG_COLLECT_OBJECT, &SC);
	if(!obLst.GetCount()) return;
	POSITION pos, ps2;
	CViecFC *pFF, *pEF;
	pos = obLst.GetHeadPosition();
	for(i=0; pos; ) {
		pFF = (CViecFC *) obLst.GetNext(pos);
		ps2 = pFF->GetHeadPosition();
		while(ps2) {
			pEF = (CViecFC *) pFF->GetNext(ps2);
			strAry.Add(pEF->GetName());
		}
	}
	if(strAry.GetSize()==0) return;
	CMenu *pSmn = GetPopupMenu(TRUE);
	pSmn->AppendMenu(MF_STRING, 0, "select one of these");
	pSmn->AppendMenu(MF_SEPARATOR);
	for(i=0; i<strAry.GetSize(); i++) {
		pSmn->AppendMenu(MF_STRING
		, CMD_SELECT_FROM_GROUP_FOR_CHILD+i, strAry[i]);
	}
	pE->TrackPopupMenu(pSmn, pE->GetMenuPoint(NULL));
}

BOOL CPDEObjPrg::OnCommandForSymbInst(int msg, CSymbInst *pI)
{
	switch(msg) {
	case CMD_CHG_SYMB_INST_TO_REAL:
		{
			CViecFC *pE = pI->GetParent();
			CMSRealEx *pR = new CMSRealEx;
			pE->SubstChild(pI, pR);
			delete pI;
			pR->OnCreated();
			pR->CalcExt(dirDown);
			pR->CalcExt();
			return TRUE;
		}
	}
	return TRUE;
}

BOOL CPDEObjPrg::AddFileToElem(CViecFC *pE, int no)	// No need for now
{
	return TRUE;
}

BOOL CPDEObjPrg::OnCommandForPad(int msg, CPad *pPad)
{
	switch(msg) {
	case CMD_NEW_FILE_INST:
		m_cmd = CMD_NEW_FILE_INST;
		ToFindCandAndSelect(pPad, RUNTIME_CLASS(CPDEFileFold));
		return TRUE;
	case CMD_PAD_NEW_BRK:
		m_cmd = CMD_PAD_NEW_BRK;
		ToFindCandAndSelect(pPad, RUNTIME_CLASS(CPDEBrkPntDefs));
		return TRUE;
	}
	/////   The result from selection  ///////
	CViecFC *pS;
	int sig2 = msg & 0xff00;
	int no = msg & 0x00ff;
	switch(sig2) {
	case CMD_SELECT_FROM_GROUP_FOR_CHILD:
		switch(m_cmd) {
		case CMD_NEW_FILE_INST:
			pS = new CPDFile;
			pS->AddTail(new CSymbInst(strAry[no]));
			pS->AddTail(new CPDArgs2);
			pPad->SetMode(modeOpen);
			pPad->AddTail(pS);
			break;
		case CMD_PAD_NEW_BRK:
			pS = new CPDBrkPnt;
			pS->AddTail(new CSymbInst(strAry[no]));
			pPad->SetMode(modeOpen);
			pPad->AddTail(pS);
			break;
		default:
			return TRUE;
		}
		pPad->OnCreated();
		pPad->CalcExt(dirDown);
		pPad->CalcExt();
	}
	return TRUE;
}

BOOL CPDEObjPrg::SendMsg(int msg, CObject* el)
{
	switch(msg) {
	case MSG_ADD_MENU_FOR_CHILD:
		{
			CSuitCase *pSC = (CSuitCase *)el;
			// m_case is the caller
			// m_case2 is the menu pointer
			if(!pSC->m_case || !pSC->m_case2) return TRUE;
			CMenu *pMenu = (CMenu *) pSC->m_case2;
			if(!pMenu->IsKindOf(RUNTIME_CLASS(CMenu))) TRUE;
			if(pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDFor))
			|| pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDProc))
			|| pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDBlock))
			) {
				CMenu *psub = GetPopupMenu(FALSE);
				pMenu->AppendMenu(MF_POPUP, (UINT)psub->m_hMenu, GetName());
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_SUBST, "add new &subst");
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_IF, "add new &if");
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_BLOCK, "add new &block");
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_FOR, "add new &for");
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_CALL, "add new &call");
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_VAR, "add new &var");
				psub->AppendMenu(MF_STRING, CMD_NEW_FILE_INST, "Add &file inst");
				psub->AppendMenu(MF_STRING, CMD_PAD_NEW_BRK, "Add &break point");
				return TRUE;
			} else if(pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDArgs2))) {
				pMenu->AppendMenu(MF_STRING, CMD_PAD_NEW_ARG2, "add new &arg");
			} else if(pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDArgs))) {
				pMenu->AppendMenu(MF_STRING, CMD_PAD_NEW_ARG, "add new &arg");
			} else if(pSC->m_case->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
				pMenu->AppendMenu(MF_STRING, CMD_CHG_SYMB_INST_TO_REAL, "add &real");
			}
			break;
		}
		return TRUE;
	case MSG_COMMAND_FOR_CHILD:
		{
			CSuitCase *pSC = (CSuitCase *) el;
			// m_int is the command id
			// m_case is the caller
			if(!pSC->m_case || !pSC->m_int) return TRUE;
			if(pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDFor))
			|| pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDProc))
			|| pSC->m_case->IsKindOf(RUNTIME_CLASS(CPDBlock))
			) {
				if(OnCommandForPad(pSC->m_int, (CPad *) pSC->m_case))
					return TRUE;
			} else if(pSC->m_case->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
				if(OnCommandForSymbInst(pSC->m_int, (CSymbInst *) pSC->m_case))
					return TRUE;
			}
			break;
		}
		return TRUE;
	}
	return CPDEBase::SendMsg(msg, el);
}

/////////////////// CPDBrkPnt /////////////////////////
/////////////////// CPDBrkPnt /////////////////////////
/////////////////// CPDBrkPnt /////////////////////////

int CPDBrkPnt::CalcExt(dirMode dir)
{
	CRect ext = GetExtent();
	CViecFC::CalcExt(dir);
	m_extent.right += s_merginX;
	m_extent.top -= s_merginY;
	m_extent.bottom += s_merginY;
	m_RName = m_extent;
	CalcExtPost(dir, ext);
	return TRUE;
}

void CPDBrkPnt::OnDraw(CDC *pdc, const CPoint& p)
{
	COLORREF col = m_bFocus? RGB(0,0,225) : RGB(0,0,0);
	CRect r = GetRect();
	r.OffsetRect(p);
	r.left += 1; r.right -=1;
	SetPen(pdc, PS_SOLID, 2, col);
	pdc->SelectStockObject(NULL_BRUSH);
	pdc->Rectangle(&r);
	CViecFC::OnDrawChild(pdc, p);
}

BOOL CPDBrkPnt::SendMsg(int msg, CObject* el)
{
	switch(msg) {
	case MSG_SIGNAL_HAS_COME:
		return TRUE;
	case MSG_BREAK_STATE_CHANGED:
		{
			CSymbInst *pBrkI = (CSymbInst *) GetChild(0);
			if(!pBrkI) return TRUE;
			if(pBrkI->GetSymbBody()!=el) return TRUE;
			InvalidateRect(CPoint(0,0));
		}
		return TRUE;
	}
	return CPad::SendMsg(msg, el);
}

/////////////////  CPDEMtxDefs  //////////////////
/////////////////  CPDEMtxDefs  //////////////////

CPDEMtxDefs::CPDEMtxDefs()
{
	SetFindFirst(TRUE);
}

int CPDEMtxDefs::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	if(GetCount()==0 || GetMode()!=modeOpen)
		return CPDEBase::CalcExt(dir);
	CalcExtLines(dir, 12, 2);

	m_extent.left -= 8;
	m_extent.right += 8;
	m_extent.top -= 6;
	m_extent.bottom += 6;
	if(dir==dirUp)
		GetParent()->CalcExt(dir);
	if(ext!=m_extent) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CPDEMtxDefs::OnDraw(CDC *pdc, const CPoint& p)
{
	DrawFold3D(pdc, p, m_bFocus? 0:1);
	if(m_mode!=modeClose && GetCount()>0) {
		OnDrawChild(pdc, p);
	} else {
		CRect r0=m_extent;
		r0.OffsetRect(m_org);
		r0.OffsetRect(p);
		CRect rD(r0.left+4, r0.top+4, r0.right-4, r0.bottom-4);
		if(m_bFocus) pdc->SetTextColor(RGB(0,0,255));
		SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
		pdc->DrawText(GetName(), lstrlen(GetName()), &rD, GetFormat());
	}
}

/////////// CPDEBrkPntDefs /////////////////////
/////////// CPDEBrkPntDefs /////////////////////

int CPDEBrkPntDefs::OnAccess(CPoint *pp)
{
	CMenu *pMenu = GetPopupMenu(TRUE);
	CString str = GetRuntimeClass()->m_lpszClassName;
	pMenu->AppendMenu(MF_STRING, 0, str+": "+GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	CMenu *psub = GetPopupMenu(FALSE);
	pMenu->AppendMenu(MF_POPUP, (UINT) psub->m_hMenu, "&default menu");
	pMenu->AppendMenu(MF_SEPARATOR);
	AppendDefaultMenu(psub);

	pMenu->AppendMenu(MF_STRING, CMD_BRK_ADD_OBJ, "&add new break object");
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

int CPDEBrkPntDefs::OnCommand(UINT id)
{
	switch(id) {
	case CMD_BRK_ADD_OBJ:
		{
			CPDEBrkObj *pBrk;
			pBrk = new CPDEBrkObj;
			pBrk->AddTail(new CPDEBttn);
			pBrk->AddTail(new CPDEBttn);
			pBrk->AddTail(new CPDEBttn);
			AddTail(pBrk);
			OnCreated();
			pBrk->CalcExt(dirDown);
			CalcExt();
		}
		return TRUE;
	}
	return CPDEBase::OnCommand(id);
}

/////////////////  CPDEBrkObj //////////////////
/////////////////  CPDEBrkObj //////////////////

CRect CPDEBrkObj::ToCalcExt(CSymbInst *ps, dirMode dir)
{
	CRect rt(0,0,0,0);
	CDC *pdc = GetDC();
	if(!pdc) return FALSE;
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	ReleaseDC(pdc);
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	UnionExtent(rt, cir);
	UnionExtent(rt, nm);
	ps->SetExtent(rt);
	return rt;
}

void CPDEBrkObj::ToDraw(CSymbInst *pInst, CDC *pdc, const CPoint& shift)
{
	CPoint p0 = shift;
	p0.Offset(-pInst->GetOrg());

	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CRect nm = GetTextExtent(pdc, GetName(), GetNameLen());
	CRect cir(nm.top, nm.top, nm.bottom, nm.bottom);
	cir.OffsetRect(CPoint(nm.bottom, 0));
	EtchingRect(cir, CRect(5,5,-5,-5));
	nm.OffsetRect(CPoint(nm.bottom * 2, 0));
	
	SelectStockObject(pdc, BLACK_PEN);
	COLORREF col;
	SetPen(pdc, PS_SOLID, 1, m_resetBtn->m_color);
	cir.OffsetRect(p0);
	pdc->MoveTo(cir.left, cir.top);
	pdc->LineTo(cir.right, cir.bottom);
	pdc->MoveTo(cir.right, cir.top);
	pdc->LineTo(cir.left, cir.bottom);
	col = pInst->m_bFocus	? RGB(0,180,0) : RGB(0,0,0);
	SetTextColor(pdc, col);
	TextOut(pdc, shift.x + nm.left, shift.y, GetName(), GetNameLen());
}

int CPDEBrkObj::CalcExt(dirMode dir)
{
	POSITION pos;
	CViecFC *pelm;
	CRect ext = m_extent;
	if(dir==dirDown) {
		pos = GetHeadPosition();
		while(pos) GetNext(pos)->CalcExt(dir);
	}
	CDC *pdc = GetDC();
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(GetName(), lstrlen(GetName()));
	ReleaseDC(pdc);
	int x = sz.cx + 15;
	pos = GetHeadPosition();
	while(pos) {
		pelm = GetNext(pos);
		pelm->SetOrg(CPoint(x, 0));
		x += pelm->GetExtent().Width() + 4;
	}
	CViecFC::CalcExt(dirNot);
	m_extent.left = 0;
	if(-(sz.cy+1)/2<m_extent.top) m_extent.top = -(sz.cy+1) / 2;
	if( (sz.cy+1)/2>m_extent.bottom) m_extent.bottom = (sz.cy+1) /2;
		
	m_extent.left += 0;
	m_extent.right += 4;
	m_extent.top += -1;
	m_extent.bottom += 1;

	if(dir==dirUp)
		GetParent()->CalcExt(dir);
	if(ext!=m_extent) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CPDEBrkObj::OnDraw(CDC *pdc, const CPoint& p)
{
	CPoint p0 = GetOrg();
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	p0.Offset(p);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(GetName(), lstrlen(GetName()));
	pdc->SetTextColor(col);
	TextOut(pdc, p0.x+5, p0.y, GetName(), GetNameLen());
	CRect rct = GetExtent();
	rct.OffsetRect(p0);
	SetPen(pdc, PS_SOLID, 1, RGB(0,0,0));
	pdc->SelectStockObject(NULL_BRUSH);
	pdc->Rectangle(&rct);
	OnDrawChild(pdc, p);
}

BOOL CPDEBrkObj::OnCreated(EnumOnCreated bSkp)
{
	if(GetCount()<3) {
		while(GetCount())
			delete RemoveTail();
		AddTail(new CPDEBttn);
		AddTail(new CPDEBttn);
		AddTail(new CPDEBttn);
	}
	POSITION pos;
	pos = GetHeadPosition();
	m_resetBtn = (CPDEBttn *) GetNext(pos);
	m_onOffBtn = (CPDEBttn *) GetNext(pos);
	m_trigBtn = (CPDEBttn *) GetNext(pos);
	return CPDEBase::OnCreated(bSkp);
}

CPDEBrkObj::CPDEBrkObj()
{
	hBrk = NULL;
}

CPDEBrkObj::~CPDEBrkObj()
{
	if(hBrk) CloseHandle(hBrk);
}

BOOL CPDEBrkObj::SendMsg(int msg, CObject* el)
{
	CPDEObjPrg *pPrg;

	switch(msg) {
	case MSG_BTTN_PRESSED:
		if(!(pPrg = (CPDEObjPrg *) GetBaseObj())) return TRUE;
		if(el==m_resetBtn) {
		} else if(el==m_onOffBtn) {
			pPrg->SendMsg(MSG_BREAK_STATE_CHANGED, this);
			if(m_bBreak) {
				m_bBreak = FALSE;
				EnterCriticalSection(&pPrg->memSync);
				if(pPrg->hVar && hBrk && pPrg->m_bBreaking && pPrg->m_pBrkObj==this) {
					pPrg->m_pBrkPnt = NULL;
					pPrg->m_pBrkObj = NULL;
					SetEvent(hBrk);
				}
				LeaveCriticalSection(&pPrg->memSync);
			} else {
				m_bBreak = TRUE;
			}
		} else if(el==m_trigBtn) {
			EnterCriticalSection(&pPrg->memSync);
			if(pPrg->hVar && hBrk && pPrg->m_bBreaking && pPrg->m_pBrkObj==this) {
				pPrg->m_pBrkPnt = NULL;
				pPrg->m_pBrkObj = NULL;
				SetEvent(hBrk);
			}
			LeaveCriticalSection(&pPrg->memSync);
		}
		if(!m_bBreak)
			m_resetBtn->m_color = RGB(0,255,0);
		else
			m_resetBtn->m_color = RGB(255,0,0);
		InvalidateRect(CPoint(0,0));
		return TRUE;
	case MSG_PROG_START_RUNNIG:
		{
			if(!(pPrg = (CPDEObjPrg *) GetBaseObj())) return TRUE;
			char buf[30];
			sprintf(buf, "%s_BRK%s", (const char *) pPrg->GetName(), GetName());
			hBrk = OpenEvent(EVENT_ALL_ACCESS, TRUE, buf);
			if(!hBrk) return TRUE;
			m_onOffBtn->m_color = RGB(0,0,255);
			m_trigBtn->m_color = RGB(0,0,255);
			InvalidateRect(CPoint(0,0));
		}
		return TRUE;
	case MSG_PROG_STOP_RUNNIG:
		if(!(pPrg = (CPDEObjPrg *) GetBaseObj())) return TRUE;
		if(hBrk) CloseHandle(hBrk);
		m_onOffBtn->m_color = RGB(0,0,80);
		m_trigBtn->m_color = RGB(0,0,80);
		InvalidateRect(CPoint(0,0));
		hBrk = NULL;
		return TRUE;
	}
	return CPDEBase::SendMsg(msg, el);
}

/////////////////// CPDFile /////////////////////////
/////////////////// CPDFile /////////////////////////

int CPDFile::CalcExt(dirMode dir)
{
	CRect ext = GetExtent();
	if(GetCount()!=2) return FALSE;
	CSymbInst *pFI = (CSymbInst *) GetChild(0);
	CPDArgs2 *pArg = (CPDArgs2 *) GetChild(1);
	if(dir==dirDown) {
		pFI->CalcExt(dir);
		pArg->CalcExt(dir);
	}
	pFI->SetOrg(CPoint(- pFI->GetExtent().left, 0));   //file
	pArg->SetOrg(CPoint(pFI->GetExtent().Width(), 0)); //
	m_extent = CRect(0,0,0,0);
	CRect rct = pFI->GetRect();
	UnionExtent(m_extent, rct);
	rct = pArg->GetRect();
	UnionExtent(m_extent, rct);
	m_extent.right += s_merginX;
	m_extent.top -= s_merginY;
	m_extent.bottom += s_merginY;
	m_RName = m_extent;
	CalcExtPost(dir, ext);
	return TRUE;
}

void CPDFile::OnDraw(CDC *pdc, const CPoint& p)
{
	COLORREF col = m_bFocus? RGB(0,0,225) : RGB(0,0,0);
	CRect r = GetRect();
	r.OffsetRect(p);
	r.left += 1; r.right -=1;
	SetPen(pdc, PS_SOLID, 2, col);
	pdc->SelectStockObject(NULL_BRUSH);
	pdc->Rectangle(&r);
	CViecFC::OnDrawChild(pdc, p);
}

///////////////// CPDFileFrom ////////////////////
///////////////// CPDFileFrom ////////////////////
int CPDFileFrom::CalcExt(dirMode dir)
{
	CRect ext = GetExtent();
	if(GetCount()!=2) return FALSE;
	CSymbInst *pFI = (CSymbInst *) GetChild(0);
	CPDArgs2 *pArg = (CPDArgs2 *) GetChild(1);
	if(dir==dirDown) {
		pFI->CalcExt(dir);
		pArg->CalcExt(dir);
	}
	pFI->SetOrg(CPoint(- pFI->GetExtent().left, 0));   //file
	pArg->SetOrg(CPoint(pFI->GetExtent().Width(), 0)); //
	m_extent = CRect(0,0,0,0);
	CRect rct = pFI->GetRect();
	UnionExtent(m_extent, rct);
	rct = pArg->GetRect();
	UnionExtent(m_extent, rct);
	m_extent.right += s_merginX;
	m_extent.top -= s_merginY;
	m_extent.bottom += s_merginY;
	m_RName = m_extent;
	CalcExtPost(dir, ext);
	return TRUE;
}

void CPDFileFrom::OnDraw(CDC *pdc, const CPoint& p)
{
	COLORREF col = m_bFocus? RGB(0,0,225) : RGB(0,0,0);
	CRect r = GetRect();
	r.OffsetRect(p);
	r.left += 1; r.right -=1;
	SetPen(pdc, PS_SOLID, 2, col);
	pdc->SelectStockObject(NULL_BRUSH);
	pdc->Rectangle(&r);
	CViecFC::OnDrawChild(pdc, p);
}
/////////////////  CPDEConstDef  //////////////////
/////////////////  CPDEConstDef  //////////////////

int CPDEConstDef::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	CViecFC *pE = GetChild(0);
	CDC *pdc = GetDC();
	if(pE==NULL || pdc==NULL) return 0;
	if(dir==dirDown) pE->CalcExt(dir);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(GetName(), lstrlen(GetName()));
	CSize s0 = pdc->GetTextExtent("=", 1);
	ReleaseDC(pdc);
	int hg = (sz.cy+1)/2;
	if(-pE->GetExtent().top>hg) hg = -pE->GetExtent().top;
	pE->SetOrg(CPoint(sz.cx + s0.cx, 0));
	m_extent.left = 0;
	m_extent.right = sz.cx + s0.cx + pE->GetExtent().Width();
	m_extent.top = -hg;
	m_extent.bottom = hg;

	if(dir==dirUp)
		GetParent()->CalcExt(dir);
	if(ext!=m_extent) {
		CRect ext0 = m_extent;
		m_extent = ext;
		InvalidateRect(CPoint(0,0));
		m_extent = ext0;
		InvalidateRect(CPoint(0,0));
	}
	return TRUE;
}

void CPDEConstDef::OnDraw(CDC *pdc, const CPoint& p)
{
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	CSize sz = pdc->GetTextExtent(GetName(), lstrlen(GetName()));
	CPoint p0 = GetOrg();
	p0.Offset(p);
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);
	TextOut(pdc, p0.x + sz.cx, p0.y, "=", 1);
	TextOut(pdc, p0.x, p0.y, GetName(), GetNameLen());
	OnDrawChild(pdc, p);
}

/////////////////  CPDESngVarBox //////////////////
/////////////////  CPDESngVarBox //////////////////

int CPDESngVarBox::CalcExt(dirMode dir)
{
	CRect ext = m_extent;
	if(GetCount()==0 || GetMode()!=modeOpen)
		return CPDEBase::CalcExt(dir);
	CalcExtVert(dir, 2);
	m_extent.left -= 8;
	m_extent.right += 8;
	m_extent.top -= 6;
	m_extent.bottom += 6;
	CalcExtPost(dir, ext);
	return TRUE;
}

int CPDESngVarBox::OnCommand(UINT id)
{
	switch(id) {
	case CMD_PROC_SHOW_VARS:
		{
			CPDESngVar *pSng = new CPDESngVar;
			AddTail(pSng);
			OnCreated();
			CalcExt(dirDown);
			CalcExt();
		}
		return TRUE;
	}
	return CPDEBase::OnCommand(id);
}

int CPDESngVarBox::OnAccess(CPoint *pp)
{
	CString str;
	CMenu *pMenu = GetPopupMenu(TRUE);
	pMenu->AppendMenu(MF_STRING, 0, GetRuntimeClass()->m_lpszClassName +CString(": ")+ GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	CMenu *psub = GetPopupMenu(FALSE);
	pMenu->AppendMenu(MF_POPUP, (UINT)psub->m_hMenu, "&Default menu");
	AppendDefaultMenu(psub);
	pMenu->AppendMenu(MF_STRING, CMD_PROC_SHOW_VARS, "add new &var");
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

BOOL CPDESngVarBox::SendMsg(int msg, CObject* el)
{
	return CPDEBase::SendMsg(msg, el);
}

////////////////////  CPDESngVar //////////////////////
////////////////////  CPDESngVar //////////////////////

CPDESngVar::CPDESngVar()
{
	m_nVarNo = -1;
	strcpy(m_pszVarName, "?");
}

int CPDESngVar::CalcExt(dirMode dir)
{
	char buf[31];
	strcpy(buf, m_pszVarName);
	CPDEObjPrg *pPrg = (CPDEObjPrg *) GetBaseElem(RUNTIME_CLASS(CPDEObjPrg));
	if(pPrg) {
		EnterCriticalSection(&pPrg->memSync);
		if(pPrg->pVar && m_nVarNo>=0 && m_nVarNo<pPrg->pVar->VarNo) {
			if(pPrg->pVar->varList[m_nVarNo].type==3) { //VAR_TYPE_REAL
				double *pd = (double *) ((BYTE*)pPrg->pVar + pPrg->pVar->varList[m_nVarNo].pBody);
				sprintf(buf, "%s = %e", m_pszVarName, *pd);
			} else if(pPrg->pVar->varList[m_nVarNo].type==4) { //VAR_TYPE_INTEGER
				int *pi = (int *) ((BYTE*)pPrg->pVar + pPrg->pVar->varList[m_nVarNo].pBody);
				sprintf(buf, "%s = %d", m_pszVarName, *pi);
			}
		}
		LeaveCriticalSection(&pPrg->memSync);
	}
	CRect ext = m_extent;
	CDC *pdc = GetDC();
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	m_extent = GetTextExtent(pdc, buf, lstrlen(buf));
	ReleaseDC(pdc);
	CalcExtPost(dir, ext);
	return 1;
}

void CPDESngVar::OnDraw(CDC *pdc, const CPoint& p)
{
	char buf[31];
	strcpy(buf, m_pszVarName);
	CPDEObjPrg *pPrg = (CPDEObjPrg *) GetBaseElem(RUNTIME_CLASS(CPDEObjPrg));
	if(pPrg) {
		EnterCriticalSection(&pPrg->memSync);
		if(pPrg->pVar && m_nVarNo>=0 && m_nVarNo<pPrg->pVar->VarNo) {
			if(pPrg->pVar->varList[m_nVarNo].type==3) { //VAR_TYPE_REAL
				double *pd = (double *) ((BYTE*)pPrg->pVar + pPrg->pVar->varList[m_nVarNo].pBody);
				sprintf(buf, "%s = %e", m_pszVarName, *pd);
			} else if(pPrg->pVar->varList[m_nVarNo].type==4) { //VAR_TYPE_INTEGER
				int *pi = (int *) ((BYTE*)pPrg->pVar + pPrg->pVar->varList[m_nVarNo].pBody);
				sprintf(buf, "%s = %d", m_pszVarName, *pi);
			}
		}
		LeaveCriticalSection(&pPrg->memSync);
	}
	CPoint p0 = p;
	p0.Offset(GetOrg());
	COLORREF col = m_bFocus? RGB(0,0,255) : RGB(0,0,0);
	pdc->SetTextColor(col);
	SetFont(pdc, m_nDot, GetFont(), m_nCharSet, m_nItalic);
	TextOut(pdc, p0.x, p0.y, buf, lstrlen(buf));
}

int CPDESngVar::OnAccess(CPoint *pp)
{
	CString str;
	CMenu *pMenu = GetPopupMenu(TRUE);
	pMenu->AppendMenu(MF_STRING, 0, GetRuntimeClass()->m_lpszClassName +CString(": ")+ GetName());
	pMenu->AppendMenu(MF_SEPARATOR);
	pMenu->AppendMenu(MF_STRING, CMD_PROC_SELECT_SNG_VAR, "select &var");
	TrackPopupMenu(pMenu, GetMenuPoint(pp));
	return 1;
}

int CPDESngVar::OnCommand(UINT id)
{
	switch(id) {
	case CMD_PROC_SELECT_SNG_VAR:
		{
			CPDEObjPrg *pPrg = (CPDEObjPrg *) GetBaseElem(RUNTIME_CLASS(CPDEObjPrg));
			if(!pPrg) return TRUE;
			EnterCriticalSection(&pPrg->memSync);
			if(!pPrg->pVar || (m_nVarNo<0 && m_nVarNo>=pPrg->pVar->VarNo)) {
				LeaveCriticalSection(&pPrg->memSync);
				return TRUE;
			}
			int i;
			strAry.RemoveAll();
			CMenu *pMenu = GetPopupMenu(TRUE);
			pMenu->AppendMenu(MF_STRING, 0, "select single var");
			pMenu->AppendMenu(MF_SEPARATOR);
			for(i=0; i<pPrg->pVar->VarNo; i++) {
				strAry.Add(pPrg->pVar->varList[i].varName);
				if(pPrg->pVar->varList[i].type==3 || pPrg->pVar->varList[i].type==4) {
					pMenu->AppendMenu(MF_STRING, CMD_FIND_DEP_RESULT+i, strAry[i]);
				}
			}
			LeaveCriticalSection(&pPrg->memSync);
			TrackPopupMenu(pMenu, GetMenuPoint(NULL));
		}
		return TRUE;
	}
	int sig2 = id & 0xff00;
	int no = id & 0x00ff;
	switch(sig2) {
	case CMD_FIND_DEP_RESULT:
		m_nVarNo = no;
		strcpy(m_pszVarName, strAry[no]);
		CalcExt();
		InvalidateRect(CPoint(0,0));
		return TRUE;
	}
	return CViecFC::OnCommand(id);
}

BOOL CPDESngVar::SendMsg(int msg, CObject* el)
{
	switch(msg) {
	case MSG_PROG_START_RUNNIG:
	case MSG_SIGNAL_HAS_COME:
	case MSG_PROG_STOP_RUNNIG:
		CalcExt();
		InvalidateRect(CPoint(0,0));
		return TRUE;
	}
	return CViecFC::SendMsg(msg, el);
}

CJDFract2Real::CJDFract2Real()
{
	m_isUnderDepVar = FALSE;
	m_isEqtDepVar = FALSE;
};

BOOL CJDFract2Real::Do(CObject *g)
{
	if(m_isUnderDepVar){ //CMSDepVar艺̃NX͎̕^ɕϊȂB
		m_isUnderDepVar = FALSE;
		return 1;
	}
	if(g->IsKindOf(RUNTIME_CLASS(CSymbInst))) {
		CSymbInst *pInst = (CSymbInst *) g;
		if(pInst->GetSymbBody()->IsKindOf(RUNTIME_CLASS(CMSDepVar))) {
			m_isUnderDepVar = TRUE; //q̃NX̕ϊOFF
			m_isEqtDepVar = TRUE; //CMSDepVar܂ސ̐^^ɕϊON
			return 1;
		}
	} else if(g->IsKindOf(RUNTIME_CLASS(CPDForSub))) {
		m_isEqtDepVar = FALSE; //CMSDepVar܂ސ̐^^ɕϊOFF
		return 1;
	//} else if(g->IsKindOf(RUNTIME_CLASS(CPDBlock))) {
	//	return 1;
	} else if(g->IsKindOf(RUNTIME_CLASS(CPDIf))) {
		m_isEqtDepVar = FALSE; //CMSDepVar܂ސ̐^^ɕϊOFF
		return 1;
	} else if(g->IsKindOf(RUNTIME_CLASS(CPDEMtxDefs))) {
		m_isEqtDepVar = FALSE; //CMSDepVar܂ސ̐^^ɕϊOFF
		return 1;
	} else if(g->IsKindOf(RUNTIME_CLASS(CMSFract))) {
		if(m_isEqtDepVar){
			CMSFract *pFC = (CMSFract*) g;
			CMSCRealSub *real;
			CViecFC* pF = (CViecFC*) g;
			real = new CMSCRealSub;
			real->m_value = (double) pFC->m_value / (double) pFC->m_deno;
			CViecFC *pE = pF->GetParent();
			pE->SubstChild( pF, real);
			delete g;
		}
		return 1;
	}
	return 0;
}

